function AMP_tests(language)

%% Settings
param.SL=40; % sensation level

%% init
if nargin<1, param.language=0; else param.language=language; end

%% window design
addpath(genpath(cd));
scsze = get(0,'ScreenSize');
color = [0.831 0.816 0.784];
param.warncolor=[1 0.7 0.2];
height=580;
length=430;
param.height=height;
param.length=length;

pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];

param.main = figure('Name','AMP','NumberTitle','off');
set(param.main,'MenuBar','none');
set(param.main,'Color', color);
set(param.main,'Position', pos);

param.caption = uicontrol('Style', 'text', 'String', 'Adaptive Music Perception Tests', 'FontSize', 12, 'FontWeight', 'bold', 'Position', [length/2 - 250 height-80 500 40], 'BackgroundColor',color);

param.txt1 = uicontrol('Style', 'text', 'String', 'Subject', 'FontSize', 10, 'Position', [30 height-120 70 20], 'BackgroundColor',color);
param.txt1a= uicontrol('Style', 'text', 'String', 'with HA', 'FontSize', 10, 'Position', [265 height-120 55 20], 'BackgroundColor',color);
param.txt1b= uicontrol('Style', 'text', 'String', 'no HA', 'FontSize', 10, 'Position', [320 height-120 60 20], 'BackgroundColor',color);

param.box1a=uicontrol('Style','radiobutton','Value',1,'Position',[290 height-140 20 20],'tag','I','callback',@function_checkbox,'BackgroundColor',color);
param.box1b=uicontrol('Style','radiobutton','Position',[345 height-140 20 20],'tag','II','callback',@function_checkbox,'BackgroundColor',color);

param.edit1= uicontrol('Style','edit','Position', [40 height-140 200 20], 'BackgroundColor',[1 1 1], 'String', 'thomas test');

param.calibration = uicontrol('Style', 'pushbutton', 'String', 'Calibration', 'FontSize', 10, 'Position', [length/2 - 70 height-225 140 50], 'BackgroundColor',color,'Callback',@function_calibration);
param.line = uicontrol('Style', 'text', 'String', '-------------------------------------------------------', 'FontSize', 10, 'Position', [length/2 - 170 height-260 340 20], 'BackgroundColor',color);
param.meter = uicontrol('Style', 'pushbutton', 'String', 'Meter', 'FontSize', 10, 'Position', [length/2 - 70 height-325 140 50], 'BackgroundColor',color,'Callback',@function_meter);
param.harmony = uicontrol('Style', 'pushbutton', 'String', 'Harmony', 'FontSize', 10,'Position', [length/2 - 70 height-400 140 50], 'BackgroundColor',color,'Callback',@function_harmony);
param.melody = uicontrol('Style', 'pushbutton', 'String', 'Melody', 'FontSize', 10,'Position', [length/2 - 70 height-475 140 50], 'BackgroundColor',color,'Callback',@function_melody);
param.timbre = uicontrol('Style', 'pushbutton', 'String', 'Timbre', 'FontSize', 10,'Position', [length/2 - 70 height-550 140 50], 'BackgroundColor',color,'Callback',@function_timbre);

if param.language
    set(param.main,'Name','Musikwahrnehmung Tests');
    set(param.caption,'String','Musikwahrnehmung Tests');
    set(param.txt1,'String','Proband');
    set(param.txt1a,'String','mit HG');
    set(param.txt1b,'String','ohne HG');
    set(param.calibration,'String','Kalibrierung');
    set(param.meter,'String','Metrum');
    set(param.harmony,'String','Harmonie');
    set(param.melody,'String','Melodie');
end

if ~exist('randparam.mat')
    Nharm_phase = 40;
    randparam.phase = rand(1,Nharm_phase)*2*pi;
    save('randparam.mat','randparam');
end

guidata(param.main,param);

%%------------------- GUI functions ----------------------%%%%
function function_checkbox(hObject, eventdata)
param=guidata(hObject);
set(param.box1a,'Value',0);
set(param.box1b,'Value',0);
set(hObject,'Value',1);
guidata(param.main,param);

function function_calibration(hObject, eventdata)
param=guidata(hObject);
if isempty(get(param.edit1,'String'))
    set(param.edit1,'BackgroundColor',param.warncolor);    return
end
if get(param.box1a,'Value')==1
    modus='withHA';
else
    modus='noHA';
end
proband=get(param.edit1,'String');
pathnfile={proband,modus};
music_calibration(pathnfile,param.language);

function function_meter(hObject, eventdata)
param=guidata(hObject);

if isempty(get(param.edit1,'String'))
    set(param.edit1,'BackgroundColor',param.warncolor);    return
end
if get(param.box1a,'Value')==1
    modus='withHA';
else
    modus='noHA';
end
proband=get(param.edit1,'String');
pathnfile={proband,modus};

if exist(['calib\' pathnfile{1} '_' pathnfile{2} '.mat'])
    helpi=load(['calib\' pathnfile{1} '_' pathnfile{2} '.mat']);
    if isfield(helpi,'MUSIC_calib')
        meter(pathnfile,helpi.MUSIC_calib{7}+param.SL,param.language);  
    else
        set(param.calibration,'BackgroundColor',param.warncolor)
        fprintf('\n')         
        disp(['No previous calibration data for subject ' pathnfile{1} '_' pathnfile{2} '.'])          
        fprintf('Please calibrate first! Default calibration is used.\n\n') 
        meter(pathnfile,12.5,param.language);
    end
else
        set(param.calibration,'BackgroundColor',param.warncolor)
        fprintf('\n')         
        disp(['No previous calibration data for subject ' pathnfile{1} '_' pathnfile{2} '.'])          
        fprintf('Please calibrate first! Default calibration is used.\n\n')
        meter(pathnfile,12.5,param.language);
end

function function_harmony(hObject, eventdata)
param=guidata(hObject);
if isempty(get(param.edit1,'String'))
    set(param.edit1,'BackgroundColor',param.warncolor);    return
end
if get(param.box1a,'Value')==1
    modus='withHA';
else
    modus='noHA';
end
proband=get(param.edit1,'String');
pathnfile={proband,modus};
if exist(['calib\' pathnfile{1} '_' pathnfile{2} '.mat'])
    helpi=load(['calib\' pathnfile{1} '_' pathnfile{2} '.mat']);
    if isfield(helpi,'MUSIC_calib')
        harmony(pathnfile,helpi.MUSIC_calib{7}+param.SL,param.language);    % add SL and loudness equalization (calculated with Chalupper/Fastl loudness model)
    else
        set(param.calibration,'BackgroundColor',param.warncolor)
        fprintf('\n')         
        disp(['No previous calibration data for subject ' pathnfile{1} '_' pathnfile{2} '.'])          
        fprintf('Please calibrate first! Default calibration is used.\n\n')
        harmony(pathnfile,12.5,param.language);
    end
else
        set(param.calibration,'BackgroundColor',param.warncolor)
        fprintf('\n')         
        disp(['No previous calibration data for subject ' pathnfile{1} '_' pathnfile{2} '.'])          
        fprintf('Please calibrate first! Default calibration is used.\n\n') 
        harmony(pathnfile,12.5,param.language);
end


function function_melody(hObject, eventdata)
param=guidata(hObject);
if isempty(get(param.edit1,'String'))
    set(param.edit1,'BackgroundColor',param.warncolor);    return
end
if get(param.box1a,'Value')==1
    modus='withHA';
else
    modus='noHA';
end
proband=get(param.edit1,'String');
pathnfile={proband,modus};
if exist(['calib\' pathnfile{1} '_' pathnfile{2} '.mat'])
    helpi=load(['calib\' pathnfile{1} '_' pathnfile{2} '.mat']);
    if isfield(helpi,'MUSIC_calib')
        melody(pathnfile,helpi.MUSIC_calib{7}+param.SL,param.language);    % add SL and loudness equalization (calculated with Chalupper/Fastl loudness model)
    else
        set(param.calibration,'BackgroundColor',param.warncolor)
        fprintf('\n')         
        disp(['No previous calibration data for subject ' pathnfile{1} '_' pathnfile{2} '.'])          
        fprintf('Please calibrate first! Default calibration is used.\n\n') 
        melody(pathnfile,12.5,param.language);
    end
else
        set(param.calibration,'BackgroundColor',param.warncolor)
        fprintf('\n')         
        disp(['No previous calibration data for subject ' pathnfile{1} '_' pathnfile{2} '.'])          
        fprintf('Please calibrate first! Default calibration is used.\n\n') 
        melody(pathnfile,12.5,param.language);
end


function function_timbre(hObject, eventdata)
param=guidata(hObject);
if isempty(get(param.edit1,'String'))
    set(param.edit1,'BackgroundColor',param.warncolor);    return
end
if get(param.box1a,'Value')==1
    modus='withHA';
else
    modus='noHA';
end
proband=get(param.edit1,'String');
pathnfile={proband,modus};
if exist(['calib\' pathnfile{1} '_' pathnfile{2} '.mat'])
    helpi=load(['calib\' pathnfile{1} '_' pathnfile{2} '.mat']);
    if isfield(helpi,'MUSIC_calib')
        timbre(pathnfile,helpi.MUSIC_calib{7}+param.SL,param.language);    % add SL and loudness equalization (calculated with Chalupper/Fastl loudness model)
    else
        set(param.calibration,'BackgroundColor',param.warncolor)
        fprintf('\n')
        disp(['No previous calibration data for subject ' pathnfile{1} '_' pathnfile{2} '.']) 
        fprintf('Please calibrate first! Default calibration is used.\n\n')
        timbre(pathnfile,12.5,param.language);  
    end
else
        set(param.calibration,'BackgroundColor',param.warncolor)
        fprintf('\n')
        disp(['No previous calibration data for subject ' pathnfile{1} '_' pathnfile{2} '.']) 
        fprintf('Please calibrate first! Default calibration is used.\n\n')
        timbre(pathnfile,12.5,param.language);
end