function harmony_test(pathnfile, calib, language)

%% Settings
param.dif{1} = 0.64;    % difference D4 dissonance (Hz)
param.dif{2} = 4.4;     % difference D5 intonation (Hz)
param.step=[0.04, 0.4]; % step sizes for D4 and D5 (Hz, Hz)

param.reversals = 3; % number or reversals that are needed to end the adaptation
param.lastrev = 2; % number of reversals that are used to calculate the difference thresholds
param.third=0; % default: 0. To add thirds to the dyads, enter 1.


%% init
if nargin<3, language=0; end
if nargin<2, calib=0; end
if nargin<1, pathnfile={'testperson','noHA'}; end

param.pathnfile = pathnfile;
param.calib = calib;
param.language = language;


%% window design
scsze = get(0,'ScreenSize');
height = 480;
length = 560;
pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];

param.main = figure('Name','Harmony: Test','NumberTitle','off');
param.color = get(param.main,'Color');
param.color_active = [0.54 0.7 1];
param.color_activepush = [0.541 0.7 1];
set(param.main,'MenuBar','none');
set(param.main,'Position', pos);
param.caption = uicontrol('Style', 'text', 'String', 'Harmony: Test', 'FontSize', 14, 'FontWeight', 'bold', 'Position', [length/2 - 180 height-60 360 30],'BackgroundColor',param.color);

param.txt1 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'After pressing "Start", you will hear two different cadences.', 'Position', [length/2-210 height-120 420 30], 'BackgroundColor',param.color);
param.txt2 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'Please decide whether cadence 1 or 2 sounds less resolved.', 'Position', [length/2-210 height-150 420 30], 'BackgroundColor',param.color);
param.txt3 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'Confirm your selection by pressing "Continue".', 'Position', [length/2-210 height-180 420 30], 'BackgroundColor',param.color);


if param.language
    set(param.main,'Name','Harmonie: Test')
    set(param.caption,'String','Harmonie: Test')
    set(param.txt1,'String','Nachdem Sie "Start" drcken, hren Sie zwei verschiedene');
    set(param.txt2,'String','Kadenzen. Entscheiden Sie welche Kadenz unruhiger');
    set(param.txt3,'String','endet und besttigen Sie Ihre Auswahl mit "Weiter".');
end


param.pb_multi=uicontrol('Style','pushbutton','String','Start','tag','1','FontSize', 14, 'FontAngle', 'italic', 'FontWeight', 'bold','Position',[length/2-200 height-440 400 70],'Callback',@function_multi);

note=iconread('images/note.png','png');

sum_note=sum(note,3);
for i = 1:size(note,1)
    for ii=1:size(note,2)
        if sum_note(i,ii)>2.5
            note(i,ii,1:3)=NaN;
        end
    end
end

param.pb_1=uicontrol('Style','pushbutton', 'cdata', note, 'String', 'A     ', 'tag', '0', 'FontSize', 28, 'Position',[length/2-200 height-330 160 120],'Callback',@function_pb);
param.pb_2=uicontrol('Style','pushbutton', 'cdata', note, 'String', 'B     ', 'tag', '0', 'FontSize', 28, 'Position',[length/2+40 height-330 160 120],'Callback',@function_pb);
param.color_pb=get(param.pb_1,'BackgroundColor');

%% stimulus design
param.equalize =  10^(1.5/20);    % used to equalize stimuli for NH, c.f. audioplayer
param.f0 = 220;
param.scaler = [1,1];
param.Nharm1 = 6;
param.Nharm3 = round(param.Nharm1*4/5);
param.Nharm5 = round(param.Nharm1*2/3);

% calculation of standard cadence
param.dur = 1;   % Length in sec
param.fs = 44100;
load('randparam.mat')
param.phase1 = randparam.phase(1:param.Nharm1);
param.phase3 = randparam.phase(param.Nharm1+1:param.Nharm1+param.Nharm3);
param.phase5 = randparam.phase(param.Nharm1+param.Nharm3+1:param.Nharm1+param.Nharm3+param.Nharm5);
for i = 1:floor(size(param.phase3,2)/4)
    param.phase3(5*i)=param.phase1(4*i)+2/3*pi; % not included when all three coincide e.g.1500
end
for i = 1:floor(size(param.phase5,2)/2)
    param.phase5(2*i)=param.phase1(3*i)+2/3*pi;
end

param.fade_time=0.08;
param.z = zeros(round(0.2*param.fs),1);
param.roll = -2;

% calculation of standard
param.harm1 = ones(1,param.Nharm1);
for i = 2:param.Nharm1
    param.harm1(i) = 10^(param.roll*(i-1)/20);
end

param.harm3 = ones(1,param.Nharm3);
param.harm3(1) = 10^(-0.5/20);
for i = 2:param.Nharm3
    param.harm3(i) = 10^((-0.5+param.roll*5/4*(i-1))/20);
end

param.harm5 = ones(1,param.Nharm5);
param.harm5(1) = 10^(-1/20);
for i = 2:param.Nharm5
    param.harm5(i) = 10^((-1+param.roll*3/2*(i-1))/20);
end

param.t = 0:1/param.fs:param.dur-1/param.fs;
param.T = param.t'*(1:param.Nharm1);
param.T3 = param.t'*(1:param.Nharm3);
param.T5 = param.t'*(1:param.Nharm5);


C  = 0.01 * sin(param.T*2*pi*param.f0+repmat(param.phase1, size(param.T,1), 1));
C = C.*repmat(param.harm1,size(param.t,2),1);
param.C = sum(C,2);

E = 0.01 * 4/5 * sin(param.T3*2*pi*param.f0*5/4+repmat(param.phase3, size(param.T3,1), 1));
E = E.*repmat(param.harm3,size(param.t,2),1);
param.E = sum(E,2);

F = 0.01 * sin(param.T*2*pi*param.f0*4/3+repmat(param.phase1, size(param.T,1), 1));
F = F.*repmat(param.harm1,size(param.t,2),1);
param.F = sum(F,2);

G_dom = 0.01 * 2/3 * sin(param.T5*2*pi*param.f0*3/2+repmat(param.phase5, size(param.T5,1), 1));
G_dom = G_dom.*repmat(param.harm5,size(param.t,2),1);
param.G_dom = sum(G_dom,2);

G_ton = 0.01 * sin(param.T*2*pi*param.f0*3/2+repmat(param.phase1, size(param.T,1), 1));
G_ton = G_ton.*repmat(param.harm1,size(param.t,2),1);
param.G_ton = sum(G_ton,2);

A = 0.01 * 4/5 * sin(param.T3*2*pi*param.f0*5/3+repmat(param.phase3, size(param.T3,1), 1));
A = A.*repmat(param.harm3,size(param.t,2),1);
param.A = sum(A,2);

B = 0.01 * 4/5 * sin(param.T3*2*pi*param.f0*3/2*5/4+repmat(param.phase3, size(param.T3,1), 1));
B = B.*repmat(param.harm3,size(param.t,2),1);
param.B = sum(B,2);

C1 = 0.01 * 2/3 * sin(param.T5*2*pi*param.f0*2+repmat(param.phase5, size(param.T5,1), 1));
C1 = C1.*repmat(param.harm5,size(param.t,2),1);
param.C1 = sum(C1,2);

D1 = 0.01 * 2/3 * sin(param.T5*2*pi*param.f0*3/2*3/2+repmat(param.phase5, size(param.T5,1), 1));
D1 = D1.*repmat(param.harm5,size(param.t,2),1);
param.D1 = sum(D1,2);


param.SUB = param.F+param.third*param.A+param.C1;
param.DOM = param.G_ton+param.third*param.B+param.D1;
param.TON = param.C+param.third*param.E+param.G_dom;

% fade-in and -out
param.fade_samples = 1:round(param.fs*param.fade_time);
param.fade = sin(pi/2*[param.fade_samples]/param.fade_samples(end));
param.SUB(param.fade_samples) = param.SUB(param.fade_samples).*param.fade';
param.SUB(end-param.fade_samples(end)+1:end) = param.SUB(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
param.DOM(param.fade_samples) = param.DOM(param.fade_samples).*param.fade';
param.DOM(end-param.fade_samples(end)+1:end) = param.DOM(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
param.TON(param.fade_samples) = param.TON(param.fade_samples).*param.fade';
param.TON(end-param.fade_samples(end)+1:end) = param.TON(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
param.turn_observer = {0,0};
param.turn_counter = [0;0];
param.turn_values={0,0};
param.AB=[0,0];
param.expert = [0;0];
param.counter = 1;

param.playback = [randperm(2);randperm(2)];
param.answer = {0,0};

if mod(param.playback(1,1),2)
    G_adapt = 0.01 * sin(param.T5*2*pi*(param.f0*3/2+param.dif{1})+repmat(param.phase5, size(param.T5,1), 1));
    G_adapt = G_adapt.*repmat(param.harm5,size(param.t,2),1);
    param.G_adapt = sum(G_adapt,2);
    param.TON_adapt = param.C+param.third*param.E+param.G_adapt;
    param.TON_adapt(param.fade_samples) = param.TON_adapt(param.fade_samples).*param.fade';
    param.TON_adapt(end-param.fade_samples(end)+1:end) = param.TON_adapt(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
else
    C_adapt = 0.01 * sin(param.T*2*pi*(param.f0+param.dif{2})+repmat(param.phase1, size(param.T,1), 1));
    C_adapt = C_adapt.*repmat(param.harm1,size(param.t,2),1);
    param.C_adapt = sum(C_adapt,2);
    
    E_adapt = 0.01 * sin(param.T3*2*pi*(param.f0+param.dif{2})*5/4+repmat(param.phase3, size(param.T3,1), 1));
    E_adapt = E_adapt.*repmat(param.harm3,size(param.t,2),1);
    param.E_adapt = sum(E_adapt,2);
    
    G_adapt = 0.01 * sin(param.T5*2*pi*(param.f0+param.dif{2})*3/2+repmat(param.phase5, size(param.T5,1), 1));
    G_adapt = G_adapt.*repmat(param.harm5,size(param.t,2),1);
    param.G_adapt = sum(G_adapt,2);
    
    param.TON_adapt = param.C_adapt+param.third*param.E_adapt+param.G_adapt;
    % fade-in and -out
    param.TON_adapt(param.fade_samples) = param.TON_adapt(param.fade_samples).*param.fade';
    param.TON_adapt(end-param.fade_samples(end)+1:end) = param.TON_adapt(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
end

param.player=audioplayer(0,param.fs);
guidata(param.main,param)

%%%%-------------------GUI functions-----------------%%%%
function function_pb(hObject,eventdata)
param=guidata(hObject);
if ~(sum(get(param.pb_1,'BackgroundColor')==param.color_active)==3 || sum(get(param.pb_2,'BackgroundColor')==param.color_active)==3)
    set(param.pb_1,'tag','0','BackgroundColor',param.color_pb);
    set(param.pb_2,'tag','0','BackgroundColor',param.color_pb);
    set(hObject,'tag','1','BackgroundColor',param.color_activepush);
    guidata(param.main,param);
end


function function_multi(hObject,eventdata)
param=guidata(hObject);
set(param.pb_1,'BackgroundColor',param.color_pb);
set(param.pb_2,'BackgroundColor',param.color_pb);
% only first time to change the String
if strcmp(get(hObject,'String'),'Start')
    tic;
    set(hObject,'String','Continue','enable','off')
    if param.language
        set(hObject,'String','Weiter')
    end
    z = param.z;
    if  param.playback(2,1)==1
        sig1=[param.SUB; z; param.DOM; z; param.TON_adapt; z(1:round(size(z,1)/4))];
        sig2=[param.SUB; z; param.DOM; z; param.TON; z(1:round(size(z,1)/4))];
    else
        sig1=[param.SUB; z; param.DOM; z; param.TON; z(1:round(size(z,1)/4))];
        sig2=[param.SUB; z; param.DOM; z; param.TON_adapt; z(1:round(size(z,1)/4))];
    end
    sig1=sig1*10^(param.calib/20);
    sig2=sig2*10^(param.calib/20);
    if max(abs([sig1;sig2]))<9.99
        param.player=audioplayer(sig1*param.equalize,param.fs);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_active);
        pause(0.001)
        playblocking(param.player);
        set(param.pb_1,'BackgroundColor',param.color_pb);
        set(param.pb_2,'BackgroundColor',param.color_pb);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_pb);
        param.player=audioplayer(sig2*param.equalize,param.fs);
        set(param.pb_2,'BackgroundColor',param.color_active);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_1,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_multi,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
    return
end

% return if nothing is chosen
if str2num(get(param.pb_1,'tag'))==0 && str2num(get(param.pb_2,'tag'))==0
    return
end

% define whether the answer is right(1) or wrong(0)
if str2num(get(param.pb_1,'tag'))*1+str2num(get(param.pb_2,'tag'))*2==param.playback(2,param.counter)
    param.answer{param.playback(1,param.counter)}(ceil(param.counter/2)) = 1;
else
    param.answer{param.playback(1,param.counter)}(ceil(param.counter/2)) = 0;
    param.AB(str2num(get(param.pb_1,'tag'))*1+str2num(get(param.pb_2,'tag'))*2)=param.AB(str2num(get(param.pb_1,'tag'))*1+str2num(get(param.pb_2,'tag'))*2)+1;
end


%% define the delta differences for the next round; deduce and count turnarounds

switch param.playback(1,param.counter)
    case 1
        % logic to determine DIRECTION (param.turn_observer, param.turn_counter)
        if param.answer{1}(end)==0
            param.turn_observer{1}(ceil(param.counter/2)) =  1; % case -
        else
            if size(param.answer{1},2) >= 2
                if param.answer{1}(end-1)==1
                    if param.turn_observer{1}(end)==0
                        param.turn_observer{1}(ceil(param.counter/2)) = -1; % case ++
                    else
                        param.turn_observer{1}(ceil(param.counter/2)) = 0;  % case (++)+
                    end
                else
                    param.turn_observer{1}(ceil(param.counter/2)) = 0;   % case -+
                end
            else
                param.turn_observer{1}(ceil(param.counter/2)) = 0; % case +
            end
        end
        
        %logic to determine a turnaround point (-1 -1 1, 0 -1 1, 1 0 -1, -1 0 1)
        %are the only combinations of increase(1), stable(0), decrease(-1), that lead to turnarounds
        if param.counter>=5 &&(...
                (param.turn_observer{1}(ceil(param.counter/2)-2)==-1 && param.turn_observer{1}(ceil(param.counter/2)-1)==-1 && param.turn_observer{1}(ceil(param.counter/2))==1)  || ...
                (param.turn_observer{1}(ceil(param.counter/2)-2)== 0 && param.turn_observer{1}(ceil(param.counter/2)-1)==-1 && param.turn_observer{1}(ceil(param.counter/2))==1)  || ...
                (param.turn_observer{1}(ceil(param.counter/2)-2)== 1 && param.turn_observer{1}(ceil(param.counter/2)-1)== 0 && param.turn_observer{1}(ceil(param.counter/2))==-1) || ...
                (param.turn_observer{1}(ceil(param.counter/2)-2)==-1 && param.turn_observer{1}(ceil(param.counter/2)-1)== 0 && param.turn_observer{1}(ceil(param.counter/2))==1) )
            param.turn_counter(1)=param.turn_counter(1)+1;
            if param.turn_counter(1)==1
                param.turn_values{1}=param.dif{1}(end);
            else
                param.turn_values{1}=[param.turn_values{1} param.dif{1}(end)];
            end
        end
        
        % define DELTA for next round (scaler, param.dif)
        if param.turn_counter(1)==1; param.scaler(1)=0.5; end
        if param.turn_counter(1)==2; param.scaler(1)=0.5; end
        if param.turn_counter(1)>=3; param.scaler(1)=0.25; end
        
        switch param.turn_observer{1}(end)
            case -1
                if size(param.dif{1},2)>=4 % logic if stuck
                    if mean(param.dif{1}(end-3:end))<=param.scaler(1)*param.step(1)
                        if param.scaler(1)==0.25
                            param.expert(1)=1;
                        else
                            param.scaler(1)=param.scaler(1)/2;
                        end
                    end
                end
                param.dif{1} = [param.dif{1} max(param.step(1)/2*param.scaler(1),param.dif{1}(end)-param.step(1)*param.scaler(1))];
            case 0
                param.dif{1} = [param.dif{1} param.dif{1}(end)];
            case 1
                param.dif{1} = [param.dif{1} param.dif{1}(end)+param.step(1)*param.scaler(1)];
        end
        
        
    case 2
        % logic to determine DIRECTION (param.turn_observer, param.turn_counter)
        if param.answer{2}(end)==0
            param.turn_observer{2}(ceil(param.counter/2)) =  1; % case -
        else
            if size(param.answer{2},2) >= 2
                if param.answer{2}(end-1)==1
                    if param.turn_observer{2}(end)==0
                        param.turn_observer{2}(ceil(param.counter/2)) = -1; % case ++
                    else
                        param.turn_observer{2}(ceil(param.counter/2)) = 0;  % case (++)+
                    end
                else
                    param.turn_observer{2}(ceil(param.counter/2)) = 0;   % case -+
                end
            else
                param.turn_observer{2}(ceil(param.counter/2)) = 0; % case +
            end
        end
        
        %logic to determine a turnaround point (-1 -1 1, 0 -1 1, 1 0 -1, -1 0 1)
        %are the only combinations of increase(1), stable(0), decrease(-1), that lead to turnarounds
        if param.counter>=5 &&(...
                (param.turn_observer{2}(ceil(param.counter/2)-2)==-1 && param.turn_observer{2}(ceil(param.counter/2)-1)==-1 && param.turn_observer{2}(ceil(param.counter/2))==1)  || ...
                (param.turn_observer{2}(ceil(param.counter/2)-2)== 0 && param.turn_observer{2}(ceil(param.counter/2)-1)==-1 && param.turn_observer{2}(ceil(param.counter/2))==1)  || ...
                (param.turn_observer{2}(ceil(param.counter/2)-2)== 1 && param.turn_observer{2}(ceil(param.counter/2)-1)== 0 && param.turn_observer{2}(ceil(param.counter/2))==-1) || ...
                (param.turn_observer{2}(ceil(param.counter/2)-2)==-1 && param.turn_observer{2}(ceil(param.counter/2)-1)== 0 && param.turn_observer{2}(ceil(param.counter/2))==1) )
            param.turn_counter(2)=param.turn_counter(2)+1;
            if param.turn_counter(2)==1
                param.turn_values{2}=param.dif{2}(end);
            else
                param.turn_values{2}=[param.turn_values{2} param.dif{2}(end)];
            end
        end
        
        % define DELTA for next round (scaler, param.dif)
        if param.turn_counter(2)==1; param.scaler(2)=0.5; end
        if param.turn_counter(2)==2; param.scaler(2)=0.5; end
        if param.turn_counter(2)>=3; param.scaler(2)=0.25; end
        
        switch param.turn_observer{2}(end)
            case -1
                if size(param.dif{2},2)>=4 % logic if stuck
                    if mean(param.dif{2}(end-3:end))<=param.scaler(2)*param.step(2)
                        if param.scaler(2)==0.25
                            param.expert(2)=1;
                        else
                            param.scaler(2)=param.scaler(2)/2;
                        end
                    end
                end
                param.dif{2} = [param.dif{2} max(param.step(2)/2*param.scaler(2),param.dif{2}(end)-param.step(2)*param.scaler(2))];
            case 0
                param.dif{2} = [param.dif{2} param.dif{2}(end)];
            case 1
                param.dif{2} = [param.dif{2} param.dif{2}(end)+param.step(2)*param.scaler(2)];
        end
        
end

fprintf('\nNumber of reversals: \n D4 (dissonance):      %d \n D5 (intonation):      %d \n \n', param.turn_counter)
fprintf('Current values: \n D4 (dissonance):      %.2f Hz\n D5 (intonation):      %.2f Hz \n \n',param.dif{1}(end),param.dif{2}(end))
fprintf('-------------------------------------------------------------------------\n')

%% add new 2-element sequence when the counter is at the end
if mod(param.counter,2) == 0
    add = [randperm(2);zeros(1,2)];
    for ii=1:2
        loc=find(param.playback(1,end-2+1:end)==ii);
        met=param.playback(2,end-2+loc); % 1.Stelle ohne Offset
        if met==1; add(2,find(add(1,:)==ii))=2;
        else add(2,find(add(1,:)==ii))=1;
        end
    end
    param.playback=[param.playback add];
end

%% calculation and playback of next trial
param.counter=param.counter+1;

if param.playback(1,param.counter)==1
    G_adapt = 0.01 * sin(param.T5*2*pi*(param.f0*3/2+param.dif{1}(end))+repmat(param.phase5, size(param.T5,1), 1));
    G_adapt = G_adapt.*repmat(param.harm5,size(param.t,2),1);
    param.G_adapt = sum(G_adapt,2);
    
    TON_adapt = param.C+param.third*param.E+param.G_adapt;
    % fade-in and -out
    TON_adapt(param.fade_samples) = TON_adapt(param.fade_samples).*param.fade';
    TON_adapt(end-param.fade_samples(end)+1:end) = TON_adapt(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
else
    C_adapt = 0.01 * sin(param.T*2*pi*(param.f0+param.dif{2}(end))+repmat(param.phase1, size(param.T,1), 1));
    C_adapt = C_adapt.*repmat(param.harm1,size(param.t,2),1);
    param.C_adapt = sum(C_adapt,2);
    
    E_adapt = 0.01 * sin(param.T3*2*pi*(param.f0+param.dif{2}(end))*5/4+repmat(param.phase3, size(param.T3,1), 1));
    E_adapt = E_adapt.*repmat(param.harm3,size(param.t,2),1);
    param.E_adapt = sum(E_adapt,2);
    
    G_adapt = 0.01 * sin(param.T5*2*pi*(param.f0+param.dif{2}(end))*3/2+repmat(param.phase5, size(param.T5,1), 1));
    G_adapt = G_adapt.*repmat(param.harm5,size(param.t,2),1);
    param.G_adapt = sum(G_adapt,2);
    
    TON_adapt = param.C_adapt+param.third*param.E_adapt+param.G_adapt;
    % fade-in and -out
    TON_adapt(param.fade_samples) = TON_adapt(param.fade_samples).*param.fade';
    TON_adapt(end-param.fade_samples(end)+1:end) = TON_adapt(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
end
z = param.z;
set(param.pb_1,'BackgroundColor',param.color_pb,'tag','0');
set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
param.time(param.counter-1)=toc;

if (param.turn_counter(1)<param.reversals && param.expert(1)==0) || (param.turn_counter(2)<param.reversals && param.expert(2)==0)
    if  param.playback(2,param.counter)==1
        sig1=[param.SUB; z; param.DOM; z; TON_adapt; z(1:round(size(z,1)/4))];
        sig2=[param.SUB; z; param.DOM; z; param.TON; z(1:round(size(z,1)/4))];
    else
        sig1=[param.SUB; z; param.DOM; z; param.TON; z(1:round(size(z,1)/4))];
        sig2=[param.SUB; z; param.DOM; z; TON_adapt; z(1:round(size(z,1)/4))];
    end
    sig1=sig1*10^(param.calib/20);
    sig2=sig2*10^(param.calib/20);
    if max(abs([sig1;sig2]))<9.99
        param.player=audioplayer(sig1*param.equalize,param.fs);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_active);
        pause(0.001)
        playblocking(param.player);
        set(param.pb_1,'BackgroundColor',param.color_pb);
        set(param.pb_2,'BackgroundColor',param.color_pb);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_pb);
        param.player=audioplayer(sig2*param.equalize,param.fs);
        set(param.pb_2,'BackgroundColor',param.color_active);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_1,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_multi,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    
    guidata(param.main,param)
else
    % create folder if it doesn't exist
    
    help1=dir;
    help2=0;
    for ii=1:size(help1,1)
        if strcmp('results',help1(ii).name)
            help2=help2+1;
        end
    end
    if help2==0;
        mkdir('results')
    end
    
    % plot and save
    if param.expert(1)
        JND{1} = 0.25*param.step(1);
    else
        JND{1} = mean(param.turn_values{1}(end-param.lastrev+1:end));
    end
    if param.expert(2)
        JND{2} = 0.25*param.step(2);
    else
        JND{2} = mean(param.turn_values{2}(end-param.lastrev+1:end));
    end
    
    HARMONY_test={param.playback, param.answer, param.dif, param.turn_observer, param.turn_counter,...
        param.turn_values, JND, param.counter, param.time, param.AB, param.f0};
    
    if exist(['results\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'])
        helpi=load(['results\' param.pathnfile{1} '_' param.pathnfile{2} '.mat']);
        if isfield(helpi,'HARMONY_test')
            [filename, pathname]=uiputfile('*.mat','Save results as',['results\' param.pathnfile{1} '_' param.pathnfile{2} '_2.mat']);
            save([pathname '/' filename],'HARMONY_test');
        else
            save(['results\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'],'HARMONY_test','-append');
        end
    else
        save(['results\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'],'HARMONY_test');
    end
    close(param.main)
    figure, plot(param.dif{1}); title('Adaptation curve for D4 (dissonance)'), xlabel('trial'), ylabel('Hz')
    saveas(gcf,['results/' param.pathnfile{1} '_' param.pathnfile{2},'_D4'],'fig');
    figure, plot(param.dif{2}); title('Adaptation curve for D5 (intonation)'), xlabel('trial'), ylabel('Hz')
    saveas(gcf,['results/' param.pathnfile{1} '_' param.pathnfile{2},'_D5'],'fig');
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], {'Discrimination thresholds'}, 'Sheet1', 'A1');
    D1_10={'D1 (dB)', 'D2 (Hz)', 'D3 (sec)', 'D4 (Hz)', 'D5 (Hz)', 'D6 (dB)', 'D7 (dB)', 'D8 (dB/harm)', 'D9 (sec)', 'D10 (dB)'};
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], D1_10, 'Sheet1', 'A2:J2');
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], JND{1}, 'Sheet1', 'D3');
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], JND{2}, 'Sheet1', 'E3');
    fprintf('-------------------------------------------------------------------------\n')
    fprintf('Difference thresholds: \n D4 (dissonance):      %.2f Hz\n D5 (intonation):      %.2f Hz \n \n',JND{1},JND{2})
    fprintf('-------------------------------------------------------------------------\n')
end