function harmony_training(pathnfile, calib, language)

%% Settings
param.dif{1} = 0.64; %   % difference in D4 (dissonance)
param.dif{2} = 6.4; %,   % difference in D5 (intonation)

param.third=0; % 0:tone dyad;  1:chords


%% init
if nargin<3, language=0; end
if nargin<2, calib=0; end
if nargin<1, pathnfile={'testperson','noHA'}; end
    
param.pathnfile = pathnfile;
param.calib = calib;
param.language = language;

%% window design
scsze = get(0,'ScreenSize');
height = 510;
length = 560;
pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];

param.main = figure('Name','Harmony: Training','NumberTitle','off');
param.color = get(param.main,'Color');
param.color_active = [0.54 0.7 1];
param.color_activepush = [0.541 0.7 1];
param.color_right = [0.4 0.7 0.2];
param.color_wrong = [1 0.2 0.2];
set(param.main,'MenuBar','none');
set(param.main,'Position', pos);
param.caption = uicontrol('Style', 'text', 'String', 'Harmony: Training', 'FontSize', 14, 'FontWeight', 'bold', 'Position', [length/2 - 180 height-60 360 30],'BackgroundColor',param.color);

param.txt1 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'After pressing "Start", you will hear two different cadences.', 'Position', [length/2-210 height-120 420 30], 'BackgroundColor',param.color);
param.txt2 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'Please decide whether cadence 1 or 2 sounds less resolved', 'Position', [length/2-210 height-150 420 30], 'BackgroundColor',param.color);
param.txt3 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'Confirm your selection by pressing "Continue".', 'Position', [length/2-210 height-180 420 30], 'BackgroundColor',param.color);
param.txt4 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'Correct answers will be highlighted in green.', 'Position', [length/2-210 height-210 420 30], 'BackgroundColor',param.color);

if param.language
    set(param.main,'Name','Harmonie: Training');
    set(param.caption,'String','Harmonie: Training');
    set(param.txt1,'String','Nachdem Sie "Start" drcken, hren Sie zwei verschiedene');
    set(param.txt2,'String','Kadenzen. Entscheiden Sie welche Kadenz unruhiger');
    set(param.txt3,'String','endet und besttigen Sie Ihre Auswahl mit "Weiter".');
    set(param.txt4,'String','Die richtige Antwort wird grn markiert, eine falsche rot.');
end
    
    
param.pb_multi=uicontrol('Style','pushbutton','String','Start','tag','1','FontSize', 14, 'FontAngle', 'italic', 'FontWeight', 'bold','Position',[length/2-200 height-470 400 70],'Callback',@function_multi);

note=iconread('images/note.png','png');

sum_note=sum(note,3);
for i = 1:size(note,1)
    for ii=1:size(note,2)
       if sum_note(i,ii)>2.5
           note(i,ii,1:3)=NaN;
       end
    end
end

param.pb_1=uicontrol('Style','pushbutton', 'cdata', note, 'String', 'A     ', 'tag', '0', 'FontSize', 28, 'Position',[length/2-200 height-360 160 120],'Callback',@function_pb);
param.pb_2=uicontrol('Style','pushbutton', 'cdata', note, 'String', 'B     ', 'tag', '0', 'FontSize', 28, 'Position',[length/2+40 height-360 160 120],'Callback',@function_pb);
param.color_pb=get(param.pb_1,'BackgroundColor');

%% stimulus design
param.f0 = 220;
param.equalize =  10^(1.5/20);    % calibration for equal output level
param.Nharm1 = 6;
param.Nharm3= round(param.Nharm1*4/5);
param.Nharm5= round(param.Nharm1*2/3);

% calculation of standard cadence
param.dur = 1;   % Length in sec
param.fs = 44100;
load('randparam.mat')
param.phase1 = randparam.phase(1:param.Nharm1);
param.phase3 = randparam.phase(param.Nharm1+1:param.Nharm1+param.Nharm3);
param.phase5 = randparam.phase(param.Nharm1+param.Nharm3+1:param.Nharm1+param.Nharm3+param.Nharm5);
for i = 1:floor(size(param.phase3,2)/4)
    param.phase3(5*i)=param.phase1(4*i)+2/3*pi; % not included when all three coincide e.g.1500
end
for i = 1:floor(size(param.phase5,2)/2)
    param.phase5(2*i)=param.phase1(3*i)+2/3*pi;
end

param.fade_time=0.08;
param.z = zeros(round(0.2*param.fs),1);
param.roll = -2;

% calculation of standard
param.harm1 = ones(1,param.Nharm1);
for i = 2:param.Nharm1
    param.harm1(i) = 10^(param.roll*(i-1)/20);
end

param.harm3 = ones(1,param.Nharm3);
param.harm3(1) = 10^(-0.5/20);
for i = 2:param.Nharm3
    param.harm3(i) = 10^((-0.5+param.roll*5/4*(i-1))/20);
end

param.harm5 = ones(1,param.Nharm5);
param.harm5(1) = 10^(-1/20);
for i = 2:param.Nharm5
    param.harm5(i) = 10^((-1+param.roll*3/2*(i-1))/20);
end

param.t = 0:1/param.fs:param.dur-1/param.fs;
param.T = param.t'*(1:param.Nharm1);
param.T3 = param.t'*(1:param.Nharm3);
param.T5 = param.t'*(1:param.Nharm5);

    
    C  = 0.01 * sin(param.T*2*pi*param.f0+repmat(param.phase1, size(param.T,1), 1));
    C = C.*repmat(param.harm1,size(param.t,2),1);
    param.C = sum(C,2);
    
    E = 0.01 * 4/5 * sin(param.T3*2*pi*param.f0*5/4+repmat(param.phase3, size(param.T3,1), 1));
    E = E.*repmat(param.harm3,size(param.t,2),1);
    param.E = sum(E,2);
    
    F = 0.01 * sin(param.T*2*pi*param.f0*4/3+repmat(param.phase1, size(param.T,1), 1));
    F = F.*repmat(param.harm1,size(param.t,2),1);
    param.F = sum(F,2);
    
    G_dom = 0.01 * 2/3 * sin(param.T5*2*pi*param.f0*3/2+repmat(param.phase5, size(param.T5,1), 1));
    G_dom = G_dom.*repmat(param.harm5,size(param.t,2),1);
    param.G_dom = sum(G_dom,2);
    
    G_ton = 0.01 * sin(param.T*2*pi*param.f0*3/2+repmat(param.phase1, size(param.T,1), 1));
    G_ton = G_ton.*repmat(param.harm1,size(param.t,2),1);
    param.G_ton = sum(G_ton,2);
    
    A = 0.01 * 4/5 * sin(param.T3*2*pi*param.f0*5/3+repmat(param.phase3, size(param.T3,1), 1));
    A = A.*repmat(param.harm3,size(param.t,2),1);
    param.A = sum(A,2);
    
    B = 0.01 * 4/5 * sin(param.T3*2*pi*param.f0*3/2*5/4+repmat(param.phase3, size(param.T3,1), 1));
    B = B.*repmat(param.harm3,size(param.t,2),1);
    param.B = sum(B,2);
    
    C1 = 0.01 * 2/3 * sin(param.T5*2*pi*param.f0*2+repmat(param.phase5, size(param.T5,1), 1));
    C1 = C1.*repmat(param.harm5,size(param.t,2),1);
    param.C1 = sum(C1,2);
    
    D1 = 0.01 * 2/3 * sin(param.T5*2*pi*param.f0*3/2*3/2+repmat(param.phase5, size(param.T5,1), 1));
    D1 = D1.*repmat(param.harm5,size(param.t,2),1);
    param.D1 = sum(D1,2);
    
    param.SUB = param.F+param.third*param.A+param.C1;
    param.DOM = param.G_ton+param.third*param.B+param.D1;
    param.TON = param.C+param.third*param.E+param.G_dom;
    
    % fade-in and -out
    param.fade_samples = 1:round(param.fs*param.fade_time);
    param.fade = sin(pi/2*[param.fade_samples]/param.fade_samples(end));
    param.SUB(param.fade_samples) = param.SUB(param.fade_samples).*param.fade';
    param.SUB(end-param.fade_samples(end)+1:end) = param.SUB(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
    param.DOM(param.fade_samples) = param.DOM(param.fade_samples).*param.fade';
    param.DOM(end-param.fade_samples(end)+1:end) = param.DOM(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
    param.TON(param.fade_samples) = param.TON(param.fade_samples).*param.fade';
    param.TON(end-param.fade_samples(end)+1:end) = param.TON(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';


% preparation, param.playback contains order of dimensions
param.counter = 1;

param.playback = [[1 1]; randperm(2)];
param.answer = 0;

if mod(param.playback(1,1),2)
    G_adapt = 0.01 * sin(param.T5*2*pi*(param.f0*3/2+param.dif{1})+repmat(param.phase5, size(param.T5,1), 1));
    G_adapt = G_adapt.*repmat(param.harm5,size(param.t,2),1);
    param.G_adapt = sum(G_adapt,2);
    param.TON_adapt = param.C+param.third*param.E+param.G_adapt;
    param.TON_adapt(param.fade_samples) = param.TON_adapt(param.fade_samples).*param.fade';
    param.TON_adapt(end-param.fade_samples(end)+1:end) = param.TON_adapt(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
else
    C_adapt = 0.01 * sin(param.T*2*pi*(param.f0+param.dif{2})+repmat(param.phase1, size(param.T,1), 1));
    C_adapt = C_adapt.*repmat(param.harm1,size(param.t,2),1);
    param.C_adapt = sum(C_adapt,2);
    
    E_adapt = 0.01 * sin(param.T3*2*pi*(param.f0+param.dif{2})*5/4+repmat(param.phase3, size(param.T3,1), 1));
    E_adapt = E_adapt.*repmat(param.harm3,size(param.t,2),1);
    param.E_adapt = sum(E_adapt,2);
    
    G_adapt = 0.01 * sin(param.T5*2*pi*(param.f0+param.dif{2})*3/2+repmat(param.phase5, size(param.T5,1), 1));
    G_adapt = G_adapt.*repmat(param.harm5,size(param.t,2),1);
    param.G_adapt = sum(G_adapt,2);
    
    param.TON_adapt = param.C_adapt+param.third*param.E_adapt+param.G_adapt;
    % fade-in and -out
    param.TON_adapt(param.fade_samples) = param.TON_adapt(param.fade_samples).*param.fade';
    param.TON_adapt(end-param.fade_samples(end)+1:end) = param.TON_adapt(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
end

param.player=audioplayer(0,param.fs);
guidata(param.main,param);

%%%%----------------------GUI functions-------------------%%%%
function function_pb(hObject,eventdata)
param=guidata(hObject);
if ~(sum(get(param.pb_1,'BackgroundColor')==param.color_active)==3 || sum(get(param.pb_2,'BackgroundColor')==param.color_active)==3)
    set(param.pb_1,'tag','0','BackgroundColor',param.color_pb);
    set(param.pb_2,'tag','0','BackgroundColor',param.color_pb);
    set(hObject,'tag','1','BackgroundColor',param.color_activepush);
    guidata(param.main,param);
end

function function_multi(hObject,eventdata)
param=guidata(hObject);
set(param.pb_1,'BackgroundColor',param.color_pb);
set(param.pb_2,'BackgroundColor',param.color_pb);
% only first time to change the String
if strcmp(get(hObject,'String'),'Start')
    tic;
    set(hObject,'String','Continue','enable','off')
    if param.language
        set(hObject,'String','Weiter')
    end
    z = param.z;
    oct = ceil(param.playback(1,param.counter)/2);
    if  param.playback(2,param.counter)==1
        sig1=[param.SUB; z; param.DOM; z; param.TON_adapt; z(1:round(size(z,1)/4))];
        sig2=[param.SUB; z; param.DOM; z; param.TON; z(1:round(size(z,1)/4))];
    else
        sig1=[param.SUB; z; param.DOM; z; param.TON; z(1:round(size(z,1)/4))];
        sig2=[param.SUB; z; param.DOM; z; param.TON_adapt; z(1:round(size(z,1)/4))];
    end
    sig1=sig1*10^(param.calib/20);
    sig2=sig2*10^(param.calib/20);
    if max(abs([sig1;sig2]))<9.99
        param.player=audioplayer(sig1*param.equalize,param.fs);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_active);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_1,'BackgroundColor',param.color_pb);
        set(param.pb_2,'BackgroundColor',param.color_pb);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_pb);
        param.player=audioplayer(sig2*param.equalize,param.fs);
        set(param.pb_2,'BackgroundColor',param.color_active);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_1,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_multi,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
    return
end

% return if nothing is chosen
if str2num(get(param.pb_1,'tag'))==0 && str2num(get(param.pb_2,'tag'))==0
    return
end
set(hObject,'enable','off');

% define whether the answer is right(1) or wrong(0)
if str2num(get(param.pb_1,'tag'))*1+str2num(get(param.pb_2,'tag'))*2==param.playback(2,param.counter)
    param.answer(param.counter)=1;
    set(eval(['param.pb_' num2str(param.playback(2,param.counter))]),'BackgroundColor',param.color_right);
else
    param.answer(param.counter)=0;
    set(eval(['param.pb_' num2str(3-param.playback(2,param.counter))]),'BackgroundColor',param.color_wrong);
end

pause(1.5)
set(param.pb_1,'BackgroundColor',param.color_pb);
set(param.pb_2,'BackgroundColor',param.color_pb);


%% define the delta differences for the next round; deduce and count turnarounds

oct = ceil(param.playback(1,param.counter)/2);
dim = param.playback(1,param.counter);

%% add new 2-element sequence when the counter is at the end
if mod(param.counter,size(param.f0,2)*2) == 0
    add = [randperm(2);zeros(1,2)];
%     if param.playback(1,end) == add(1,1)
%         helpi=randperm(size(param.f0,2)*2-1);
%         add(1,1) = add(1,helpi(1)+1);
%         add(1,helpi(1)+1) = param.playback(1,end);
%     end
    for ii=1:2
        loc=find(param.playback(1,end-size(param.f0,2)*2+1:end)==ii);
        met=param.playback(2,end-size(param.f0,2)*2+loc); % 1.Stelle ohne Offset
        if met==1; add(2,find(add(1,:)==ii))=2;
        else add(2,find(add(1,:)==ii))=1;
        end
    end
    param.playback=[param.playback add];
end

%% calculation and playback of next trial
param.counter=param.counter+1;

oct = ceil(param.playback(1,param.counter)/2);
dim = param.playback(1,param.counter);
if mod(dim,2);
    G_adapt = 0.01 * sin(param.T5*2*pi*(param.f0*3/2+param.dif{1}(end))+repmat(param.phase5, size(param.T5,1), 1));
    G_adapt = G_adapt.*repmat(param.harm5,size(param.t,2),1);
    param.G_adapt = sum(G_adapt,2);
    
    TON_adapt = param.C+param.third*param.E+param.G_adapt;
    % fade-in and -out
    TON_adapt(param.fade_samples) = TON_adapt(param.fade_samples).*param.fade';
    TON_adapt(end-param.fade_samples(end)+1:end) = TON_adapt(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
else
    C_adapt = 0.01 * sin(param.T*2*pi*(param.f0+param.dif{2}(end))+repmat(param.phase1, size(param.T,1), 1));
    C_adapt = C_adapt.*repmat(param.harm1,size(param.t,2),1);
    param.C_adapt = sum(C_adapt,2);
    
    E_adapt = 0.01 * sin(param.T3*2*pi*(param.f0+param.dif{2}(end))*5/4+repmat(param.phase3, size(param.T3,1), 1));
    E_adapt = E_adapt.*repmat(param.harm3,size(param.t,2),1);
    param.E_adapt = sum(E_adapt,2);
    
    G_adapt = 0.01 * sin(param.T5*2*pi*(param.f0+param.dif{2}(end))*3/2+repmat(param.phase5, size(param.T5,1), 1));
    G_adapt = G_adapt.*repmat(param.harm5,size(param.t,2),1);
    param.G_adapt = sum(G_adapt,2);
    
    TON_adapt = param.C_adapt+param.third*param.E_adapt+param.G_adapt;
    % fade-in and -out
    TON_adapt(param.fade_samples) = TON_adapt(param.fade_samples).*param.fade';
    TON_adapt(end-param.fade_samples(end)+1:end) = TON_adapt(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';
end  
    z = param.z;
    if  param.playback(2,param.counter)==1
        sig1=[param.SUB; z; param.DOM; z; TON_adapt; z(1:round(size(z,1)/4))];
        sig2=[param.SUB; z; param.DOM; z; param.TON; z(1:round(size(z,1)/4))];
    else
        sig1=[param.SUB; z; param.DOM; z; param.TON; z(1:round(size(z,1)/4))];
        sig2=[param.SUB; z; param.DOM; z; TON_adapt; z(1:round(size(z,1)/4))];
    end
param.time(param.counter-1)=toc;
if param.counter<=6
    sig1=sig1*10^(param.calib/20);
    sig2=sig2*10^(param.calib/20);
if max(abs([sig1;sig2]))<9.99
        param.player=audioplayer(sig1*param.equalize,param.fs);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_active);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_1,'BackgroundColor',param.color_pb);
        set(param.pb_2,'BackgroundColor',param.color_pb);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_pb);
        param.player=audioplayer(sig2*param.equalize,param.fs);
        set(param.pb_2,'BackgroundColor',param.color_active);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_1,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_multi,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
else
    % create folder if it doesn't exist
    
    help1=dir;
    help2=0;
    for ii=1:size(help1,1)
        if strcmp('results',help1(ii).name)
            help2=help2+1;
        end
    end
    if help2==0;
        mkdir('results')
    end
    
    close(param.main)
   harmony_test(param.pathnfile, param.calib, param.language)
end