function melody_training(pathnfile, calib, language)

%% Settings
param.dif{1} = 0;     % difference D6 Melody-to-Chord Ratio I  (dB)
param.dif{2} = 0;     % difference D7 Melody-to-Chord Ratio II (dB)

%% init
if nargin<3, language=1; end
if nargin<2, calib=0; end
if nargin<1, pathnfile={'testperson','noHA'}; end


param.pathnfile = pathnfile;
param.calib = calib;
param.language = language;

%% window
scsze = get(0,'ScreenSize');
height = 510;
length = 600;
pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];

param.main = figure('Name','Melody: Training','NumberTitle','off');
param.color = get(param.main,'Color');
param.color_active = [0.54 0.7 1];
param.color_activepush = [0.541 0.7 1];
param.color_right = [0.4 0.7 0.2];
param.color_wrong = [1 0.2 0.2];
set(param.main,'MenuBar','none');
set(param.main,'Position', pos);
param.caption = uicontrol('Style', 'text', 'String', 'Melody: Training', 'FontSize', 14, 'FontWeight', 'bold', 'Position', [length/2 - 180 height-60 360 30],'BackgroundColor',param.color);

param.txt1 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'After pressing "Start", you will hear two chords that are accompanied', 'Position', [length/2-240 height-120 480 30], 'BackgroundColor',param.color);
param.txt2 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'by melodies. Please decide whether the contours of the melodies are', 'Position', [length/2-240 height-150 480 30], 'BackgroundColor',param.color);
param.txt3 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'equal or not and confirm your selection by pressing "Continue".', 'Position', [length/2-240 height-180 480 30], 'BackgroundColor',param.color);
param.txt4 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'The correct answer will be highlighted in green.', 'Position', [length/2-240 height-210 480 30], 'BackgroundColor',param.color);

if param.language
    set(param.main,'Name','Melodie: Training')
    set(param.caption,'String','Melodie: Training')
    set(param.txt1,'String','Nachdem Sie "Start" drcken, hren sie zwei Akkorde, die mit Melo-');
    set(param.txt2,'String','dien verziert sind. Entscheiden Sie, ob die Melodieverlufe gleich');
    set(param.txt3,'String','oder ungleich sind und besttigen Sie Ihre Auswahl mit "Weiter".');
    set(param.txt4,'String','Die richtige Antwort wird grn markiert.');
end

param.pb_multi=uicontrol('Style','pushbutton','String','Start','tag','1','FontSize', 14, 'FontAngle', 'italic', 'FontWeight', 'bold','Position',[length/2-200 height-470 400 70],'Callback',@function_multi);

note=iconread('images/note.png','png');

sum_note=sum(note,3);
for i = 1:size(note,1)
    for ii=1:size(note,2)
        if sum_note(i,ii)>2.5
            note(i,ii,1:3)=NaN;
        end
    end
end

param.pb_1=uicontrol('Style','pushbutton', 'cdata', note, 'String', 'equal               ', 'tag', '0', 'FontSize', 14, 'Position',[length/2-200 height-360 160 120],'Callback',@function_pb);
param.pb_2=uicontrol('Style','pushbutton', 'cdata', note, 'String', 'not equal             ', 'tag', '0', 'FontSize', 14, 'Position',[length/2+40 height-360 160 120],'Callback',@function_pb);

if param.language
    set(param.pb_1,'String','gleich             ')
    set(param.pb_2,'String','ungleich            ')
end

param.color_pb=get(param.pb_1,'BackgroundColor');

%% stimulus design
param.equalize =  10^(-7/20);   % used to equalize stimuli for NH, c.f. audioplayer
param.fs=44100;
param.Nharm = 40;
param.roll = -2;
param.fade_time=0.08;

param.dur=3; % sec
param.f0 = 220;
param.scaler = [1,1];

load('randparam.mat')
param.phase = randparam.phase(1:param.Nharm);

%% calculation of chords
param.harm = ones(1,param.Nharm);
for i = 2:param.Nharm
    param.harm(i) = 10^(param.roll*(i-1)/20);
end

param.t = 0:1/param.fs:param.dur-1/param.fs;
param.T = param.t'*(1:param.Nharm);
param.t6 = 0:1/param.fs:param.dur/6-1/param.fs;
param.T6 = param.t6'*(1:param.Nharm);
param.z=zeros(param.dur*param.fs/3,1);

% chord 1
C1  = 0.01 * sin(param.T*2*pi*param.f0+repmat(param.phase, size(param.T,1), 1));
C1 = C1.*repmat(param.harm,size(param.t,2),1);
param.C1 = sum(C1,2);

G1  = 0.01 * sin(param.T*2*pi*param.f0*(2^(7/12))+repmat(param.phase, size(param.T,1), 1));
G1 = G1.*repmat(param.harm,size(param.t,2),1);
param.G1 = sum(G1,2);

E2  = 0.01 * sin(param.T*2*pi*param.f0*(2^(16/12))+repmat(param.phase, size(param.T,1), 1));
E2 = E2.*repmat(param.harm,size(param.t,2),1);
param.E2 = sum(E2,2);

% chord 2
B  = 0.01 * sin(param.T*2*pi*param.f0*(2^(-2/12))+repmat(param.phase, size(param.T,1), 1));
B = B.*repmat(param.harm,size(param.t,2),1);
param.B = sum(B,2);

F1  = 0.01 * sin(param.T*2*pi*param.f0*(2^(5/12))+repmat(param.phase, size(param.T,1), 1));
F1 = F1.*repmat(param.harm,size(param.t,2),1);
param.F1 = sum(F1,2);

D2  = 0.01 * sin(param.T*2*pi*param.f0*(2^(14/12))+repmat(param.phase, size(param.T,1), 1));
D2 = D2.*repmat(param.harm,size(param.t,2),1);
param.D2 = sum(D2,2);

param.CH1 = param.C1+param.G1+param.E2;
param.CH2 = param.B+param.F1+param.D2;

% melody tones set 1
d1  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(2/12))+repmat(param.phase, size(param.T6,1), 1));
d1 = d1.*repmat(param.harm,size(param.t6,2),1);
param.d1 = sum(d1,2);

e1  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(4/12))+repmat(param.phase, size(param.T6,1), 1));
e1 = e1.*repmat(param.harm,size(param.t6,2),1);
param.e1 = sum(e1,2);

f1  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(5/12))+repmat(param.phase, size(param.T6,1), 1));
f1 = f1.*repmat(param.harm,size(param.t6,2),1);
param.f1 = sum(f1,2);

a1  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(9/12))+repmat(param.phase, size(param.T6,1), 1));
a1 = a1.*repmat(param.harm,size(param.t6,2),1);
param.a1 = sum(a1,2);

h1  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(11/12))+repmat(param.phase, size(param.T6,1), 1));
h1 = h1.*repmat(param.harm,size(param.t6,2),1);
param.h1 = sum(h1,2);

d2  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(14/12))+repmat(param.phase, size(param.T6,1), 1));
d2 = d2.*repmat(param.harm,size(param.t6,2),1);
param.d2= sum(d2,2);

% melody tones set 2

c1  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(0/12))+repmat(param.phase, size(param.T6,1), 1));
c1 = c1.*repmat(param.harm,size(param.t6,2),1);
param.c1 = sum(c1,2);

%d1

es1  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(3/12))+repmat(param.phase, size(param.T6,1), 1));
es1 = es1.*repmat(param.harm,size(param.t6,2),1);
param.es1 = sum(es1,2);

g1  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(7/12))+repmat(param.phase, size(param.T6,1), 1));
g1 = g1.*repmat(param.harm,size(param.t6,2),1);
param.g1 = sum(g1,2);

%a1

c2  = 0.01 * sin(param.T6*2*pi*param.f0*(2^(12/12))+repmat(param.phase, size(param.T6,1), 1));
c2 = c2.*repmat(param.harm,size(param.t6,2),1);
param.c2= sum(c2,2);

% fade-in and -out
param.fade_samples = 1:round(param.fs*param.fade_time);
param.fade = sin(pi/2*[param.fade_samples]/param.fade_samples(end));

param.CH1(param.fade_samples) = param.CH1(param.fade_samples).*param.fade';
param.CH1(end-param.fade_samples(end)+1:end) = param.CH1(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.CH2(param.fade_samples) = param.CH2(param.fade_samples).*param.fade';
param.CH2(end-param.fade_samples(end)+1:end) = param.CH2(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.d1(param.fade_samples) = param.d1(param.fade_samples).*param.fade';
param.d1(end-param.fade_samples(end)+1:end) = param.d1(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.e1(param.fade_samples) = param.e1(param.fade_samples).*param.fade';
param.e1(end-param.fade_samples(end)+1:end) = param.e1(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.f1(param.fade_samples) = param.f1(param.fade_samples).*param.fade';
param.f1(end-param.fade_samples(end)+1:end) = param.f1(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.a1(param.fade_samples) = param.a1(param.fade_samples).*param.fade';
param.a1(end-param.fade_samples(end)+1:end) = param.a1(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.h1(param.fade_samples) = param.h1(param.fade_samples).*param.fade';
param.h1(end-param.fade_samples(end)+1:end) = param.h1(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.d2(param.fade_samples) = param.d2(param.fade_samples).*param.fade';
param.d2(end-param.fade_samples(end)+1:end) = param.d2(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.c1(param.fade_samples) = param.c1(param.fade_samples).*param.fade';
param.c1(end-param.fade_samples(end)+1:end) = param.c1(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.es1(param.fade_samples) = param.es1(param.fade_samples).*param.fade';
param.es1(end-param.fade_samples(end)+1:end) = param.es1(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.g1(param.fade_samples) = param.g1(param.fade_samples).*param.fade';
param.g1(end-param.fade_samples(end)+1:end) = param.g1(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.c2(param.fade_samples) = param.c2(param.fade_samples).*param.fade';
param.c2(end-param.fade_samples(end)+1:end) = param.c2(end-param.fade_samples(end)+1:end).*param.fade(end:-1:1)';

param.MEL1 = [param.d1,param.e1,param.f1,param.a1,param.h1,param.d2];
param.MEL2 = [param.c1,param.d1,param.es1,param.g1,param.a1,param.c2];

param.melodies=[2,1,3,2; 2,3,1,2; 5,4,6,5; 5,6,4,5];

param.turn_observer = {[],[]};
param.answer = {[],[]};
param.turn_counter = [0;0];
param.turn_values={0,0};
param.AB=[0,0];
param.counter = 1;

% construct param.playback / logic for 2nd row: alternately 1,2 within each group (group1=(1,2) and
% group2=(3,4))
helpi=[1,2,3,4;1 2 1 2];
helpi2=randperm(4);
param.playback = zeros(2,4);
for i=1:4
    param.playback(:,i)=helpi(:,helpi2(i));
end
   
param.player=audioplayer(0,param.fs);
guidata(param.main,param)

%%%%-----------------------GUI functions---------------------%%%%
function function_pb(hObject,eventdata)
param=guidata(hObject);
if ~(sum(get(param.pb_1,'BackgroundColor')==param.color_active)==3 || sum(get(param.pb_2,'BackgroundColor')==param.color_active)==3)
    set(param.pb_1,'tag','0','BackgroundColor',param.color_pb);
    set(param.pb_2,'tag','0','BackgroundColor',param.color_pb);
    set(hObject,'tag','1','BackgroundColor',param.color_activepush);
    guidata(param.main,param);
end

function function_multi(hObject,eventdata)
param=guidata(hObject);
set(param.pb_1,'BackgroundColor',param.color_pb);
set(param.pb_2,'BackgroundColor',param.color_pb);

% only first time to change the String
if strcmp(get(hObject,'String'),'Start')
    tic;
    set(hObject,'String','Continue','enable','off')
    if param.language
        set(hObject,'String','Weiter')
    end
    mel=param.playback(1,param.counter);
    sig1=1*param.CH1 + 10^(param.dif{ceil(mel/2)}(end)/20)*[param.z;param.MEL1(:,param.melodies(mel,1));param.MEL1(:,param.melodies(mel,2));param.MEL1(:,param.melodies(mel,3));param.MEL1(:,param.melodies(mel,4))];
    if param.playback(2,param.counter)==2 %change melody
    if mel<=2
    mel=3-mel;
    else
        mel=7-mel;
    end
    end
    sig2=1*param.CH2 + 10^(param.dif{ceil(mel/2)}(end)/20)*[param.z;param.MEL2(:,param.melodies(mel,1));param.MEL2(:,param.melodies(mel,2));param.MEL2(:,param.melodies(mel,3));param.MEL2(:,param.melodies(mel,4))];
    
    sig1=sig1*10^(param.calib/20)*param.equalize;
    sig2=sig2*10^(param.calib/20)*param.equalize;
    
if max(abs([sig1;sig2]))<0.99
    pause(0.01)
    param.player=audioplayer([sig1],param.fs);

        playblocking(param.player);
            pause(1.1)
param.player=audioplayer([sig2],param.fs);
        playblocking(param.player);
        pause(0.001)
        set(param.pb_multi,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
    return
end

% return if nothing is chosen
if str2num(get(param.pb_1,'tag'))==0 && str2num(get(param.pb_2,'tag'))==0
    return
end

% define whether the answer is right(1) or wrong(0)
if str2num(get(param.pb_1,'tag'))*1+str2num(get(param.pb_2,'tag'))*2==param.playback(2,param.counter)
    if isempty(param.answer{ceil(param.playback(1,param.counter)/2)})
        param.answer{ceil(param.playback(1,param.counter)/2)}(1)=1;
    else
    param.answer{ceil(param.playback(1,param.counter)/2)} = [param.answer{ceil(param.playback(1,param.counter)/2)}, 1];
    end
      set(eval(['param.pb_' num2str(param.playback(2,param.counter))]),'BackgroundColor',param.color_right);
else
    if isempty(param.answer{ceil(param.playback(1,param.counter)/2)})
        param.answer{ceil(param.playback(1,param.counter)/2)}(1)=0;
    else
    param.answer{ceil(param.playback(1,param.counter)/2)} = [param.answer{ceil(param.playback(1,param.counter)/2)}, 0];
    param.AB(str2num(get(param.pb_1,'tag'))*1+str2num(get(param.pb_2,'tag'))*2)=param.AB(str2num(get(param.pb_1,'tag'))*1+str2num(get(param.pb_2,'tag'))*2)+1;
    end
      set(eval(['param.pb_' num2str((3-param.playback(2,param.counter)))]),'BackgroundColor',param.color_wrong);
end

pause(1.4)
set(param.pb_1,'BackgroundColor',param.color_pb);
set(param.pb_2,'BackgroundColor',param.color_pb);

%% add new 2-element sequence when the counter is at the end
if mod(param.counter,4) == 0
    add = [randperm(4);zeros(1,4)];
    if param.playback(1,end) == add(1,1)  % so that consecutive trials are different
        helpi=randperm(2-1);
        add(1,1) = add(1,helpi(1)+1);
        add(1,helpi(1)+1) = param.playback(1,end);
    end
    for ii=1:4
        loc=find(param.playback(1,end-4+1:end)==ii);
        met=param.playback(2,end-4+loc); % 1.Position without offset
        if met==1; add(2,find(add(1,:)==ii))=2;
        else add(2,find(add(1,:)==ii))=1;
        end
    end
    param.playback=[param.playback add];
end

%% calculation and playback of next trial
param.counter=param.counter+1;
z = param.z;
set(param.pb_1,'BackgroundColor',param.color_pb,'tag','0');
set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
param.time(param.counter-1)=toc;

if param.counter<=6

  tic;
    set(hObject,'String','Continue','enable','off')
    if param.language
        set(hObject,'String','Weiter')
    end
    mel=param.playback(1,param.counter);
    sig1=1*param.CH1 + 10^(param.dif{ceil(mel/2)}(end)/20)*[param.z;param.MEL1(:,param.melodies(mel,1));param.MEL1(:,param.melodies(mel,2));param.MEL1(:,param.melodies(mel,3));param.MEL1(:,param.melodies(mel,4))];
    if param.playback(2,param.counter)==2 %change melody
    if mel<=2
    mel=3-mel;
    else
        mel=7-mel;
    end
    end
    sig2=1*param.CH2 + 10^(param.dif{ceil(mel/2)}(end)/20)*[param.z;param.MEL2(:,param.melodies(mel,1));param.MEL2(:,param.melodies(mel,2));param.MEL2(:,param.melodies(mel,3));param.MEL2(:,param.melodies(mel,4))];
    
    sig1=sig1*10^(param.calib/20)*param.equalize;
    sig2=sig2*10^(param.calib/20)*param.equalize;
    
if max(abs([sig1;sig2]))<0.99
    pause(0.01)
    param.player=audioplayer([sig1],param.fs);
        playblocking(param.player);
    pause(1.1)
param.player=audioplayer([sig2],param.fs);
        playblocking(param.player);
        pause(0.001)
        set(param.pb_multi,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
end

    guidata(param.main,param)
else
    % create folder if it doesn't exist
    help1=dir;
    help2=0;
    for ii=1:size(help1,1)
        if strcmp('results',help1(ii).name)
            help2=help2+1;
        end
    end
    if help2==0;
        mkdir('results')
    end
    
    close(param.main)
end