function meter_test(pathnfile,calib,language)

%% Settings
param.dif{1} = 4;%      difference in D1 level (dB)
param.dif{2} = 4;%      difference in D2 pitch (Hz)
param.dif{3} = 0.15; %  difference in D3 duration (sec)
param.step = [0.75,0.6,0.02]; % step sizes for D1, D2, and D3 (dB, Hz, sec)
param.reversals = 3; % number or reversals that are needed to end the adaptation
param.lastrev = 2; % number of reversals that are used to calculate the difference thresholds

%% init
if nargin<3, language=0; end
if nargin<2, calib=0; end
if nargin<1, pathnfile={'thomas test','withHA'}; end

param.pathnfile=pathnfile;
param.calib=calib;
param.language=language;

% param.answer: Answers for all trials, 1 right, 0 wrong
% param.playback: 1st row = the dimension under test (1:lev, 2:f0,
% 3:dur); 2nd row = the played back meter of that trial
% param.dif{1}, ..f0_dif, ..dur_dif: show the history of the
% differences that have been under test for the particular dimension

%% window design
scsze = get(0,'ScreenSize');
height = 470;
length = 480;
pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];

param.main = figure('Name','Meter: Test','NumberTitle','off');
param.color=get(param.main,'Color');
param.color_active = [0.54 0.7 1];
set(param.main,'MenuBar','none');
set(param.main,'Position', pos);
param.caption = uicontrol('Style', 'text', 'String', 'Meter: Test', 'FontSize', 14, 'FontWeight', 'bold', 'Position', [length/2 - 180 height-60 360 30],'BackgroundColor',param.color);


param.txt1 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'After pressing "Start", you will hear a sequence of', 'Position', [length/2-210 height-120 420 30], 'BackgroundColor',param.color);
param.txt2 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'beats. Please decide whether it is a 2/4 or 3/4 me-', 'Position', [length/2-210 height-150 420 30], 'BackgroundColor',param.color);
param.txt3 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'ter. Confirm your selection by pressing "Continue".', 'Position', [length/2-210 height-180 420 30], 'BackgroundColor',param.color);

param.pb_multi=uicontrol('Style','pushbutton','String','Start','tag','1','FontSize', 14, 'FontAngle', 'italic', 'FontWeight', 'bold','Position',[length/2-160 height-430 320 60],'Callback',@function_multi);

if param.language
    set(param.main,'Name','Metrum: Test');
    set(param.caption,'String','Metrum: Test');
    set(param.txt1,'String','Nachdem Sie "Start" drcken, hren Sie eine Tonfolge.');
    set(param.txt2,'String','Entscheiden Sie, ob Sie die Tne im 2/4 oder 3/4 Takt');
    set(param.txt3,'String','wahrnehmen. Besttigen Sie Ihre Antwort mit "Weiter".')
end


note=iconread('images/note.png','png');

sum_note=sum(note,3);
for i = 1:size(note,1)
    for ii=1:size(note,2)
        if sum_note(i,ii)>2.5
            note(i,ii,1:3)=NaN;
        end
    end
end

param.pb_2=uicontrol('Style','pushbutton', 'String', '2/4    ', 'cdata',note, 'tag','0', 'FontSize', 24, 'Position',[length/2-160 height-330 120 120],'Callback',@function_pb);
param.pb_3=uicontrol('Style','pushbutton', 'String', '3/4    ', 'cdata',note, 'tag','0', 'FontSize', 24, 'Position',[length/2+40 height-330 120 120],'Callback',@function_pb);
param.color_pb=get(param.pb_3,'BackgroundColor');

%% stimulus design
param.fs=44100;
param.bpm = 100;
param.dur = 0.28;
param.f0 = 220;
param.Nharm = 40;
param.roll = -2;
param.fade_time=0.08;

param.scaler=[1,1,1];

% phase
param.Nharm = 40;
t = 0:1/param.fs:param.dur-1/param.fs;
T = t'*(1:param.Nharm);

%% calculation of the standard upbeat
load('randparam.mat');
param.phase=randparam.phase(1:param.Nharm);
UP = 0.01 * sin(T*2*pi*param.f0+repmat(param.phase,size(T,1),1));
harm = ones(1,param.Nharm);
roll = 10^(param.roll/20);
for i = 2:param.Nharm
    harm(i) = roll^(i-1);
end
UP = UP.*repmat(harm,size(t,2),1);
up = sum(UP,2);
up = up * 10^(param.calib/20);

% fade-in and -out
fade_samples = 1:round(param.fs*param.fade_time);
fade = sin(pi/2*[fade_samples]/fade_samples(end));
up(fade_samples) = up(fade_samples).*fade';
up(end-fade_samples(end)+1:end) = up(end-fade_samples(end)+1:end).*fade(end:-1:1)';

%fill in zeros
param.up=[up; zeros(round(60*param.fs/param.bpm)-size(up,1),1)];

param.counter=1;
param.playback=[randperm(3);...
    round(randperm(3)/1.5)+1];
param.answer={0,0,0};
param.turn_observer={0,0,0};
param.expert=[0;0;0];
param.turn_counter=[0;0;0];
param.turn_values={0,0,0};
param.AB=[0,0];

%% preparation of downbeat for first round
if param.playback(1,param.counter)==3
    t = 0:1/param.fs:param.dur+param.dif{3}(end)-1/param.fs;
end
T = t'*(1:param.Nharm);
if param.playback(1,param.counter)==2
    DOWN = 0.01 * sin(T*2*pi*(param.f0+param.dif{2}(end))+repmat(param.phase,size(T,1),1));
else DOWN = 0.01 * sin(T*2*pi*param.f0+repmat(param.phase,size(T,1),1)); end
harm = ones(1,param.Nharm);
roll = 10^(param.roll/20);
for i = 2:param.Nharm
    harm(i) = roll^(i-1);
end
DOWN = DOWN.*repmat(harm,size(t,2),1);
if param.playback(1,param.counter)==1
    down = sum(DOWN,2);
    down = down * 10^((param.calib+param.dif{1}(end))/20);
else down = sum(DOWN,2)*10^((param.calib)/20); end

% fade-in and -out
fade_samples = 1:round(param.fs*param.fade_time);
fade = sin(pi/2*[fade_samples]/fade_samples(end));
down(fade_samples) = down(fade_samples).*fade';
down(end-fade_samples(end)+1:end) = down(end-fade_samples(end)+1:end).*fade(end:-1:1)';

%fill in zeros
param.down=[down; zeros(round(60*param.fs/param.bpm)-size(down,1),1)];
param.zeros=round(60*param.fs/param.bpm)-size(down,1);

%% preparation, param.playback contains order of dimensions (1= level,
%% 2=pitch, 3=duration, 4=roll off) and the played back meter (3 or 2)

guidata(param.main,param)

function function_pb(hObject,eventdata)
param=guidata(hObject);
set(param.pb_2,'tag','0','BackgroundColor',param.color_pb,'tag','0');
set(param.pb_3,'tag','0','BackgroundColor',param.color_pb,'tag','0');
set(hObject,'tag','1','BackgroundColor',param.color_active);
guidata(param.main,param);

function function_multi(hObject,eventdata)
param=guidata(hObject);

% only first time to change the String
if strcmp(get(hObject,'String'),'Start')
    tic;
    set(hObject,'String','Continue','enable','off')
    if param.language
        set(hObject,'String','Weiter')
    end
    set(param.pb_2,'BackgroundColor',param.color_pb);
    set(param.pb_3,'BackgroundColor',param.color_pb);
    sig=repmat([param.down; repmat(param.up,param.playback(2,param.counter)-1,1)],5-param.playback(2,param.counter),1);
    sig=sig(1:end-param.zeros);
    if max(abs(sig))<0.99
        pause(0.5)
        set(param.pb_2,'BackgroundColor',param.color_pb);
        set(param.pb_3,'BackgroundColor',param.color_pb);
        param.player=audioplayer(sig,param.fs);
        pause(0.001)
        playblocking(param.player);
        pause(0.01)
        set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_3,'BackgroundColor',param.color_pb,'tag','0');
        set(hObject,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
    return
end

stop(param.player)

%% return if no answer is selected ( not needed as "Confirm button" is
%% desactivated

if strcmp(get(param.pb_2,'tag'),get(param.pb_3,'tag')) % both 0
    return
end

% define whether the answer is right(1) or wrong(0)
if str2num(get(param.pb_2,'tag'))*2+str2num(get(param.pb_3,'tag'))*3==param.playback(2,param.counter)
    param.answer{param.playback(1,param.counter)}(ceil(param.counter/3)) = 1;
else
    param.answer{param.playback(1,param.counter)}(ceil(param.counter/3)) = 0;
    param.AB(str2num(get(param.pb_2,'tag'))*1+str2num(get(param.pb_3,'tag'))*2)=param.AB(str2num(get(param.pb_2,'tag'))*1+str2num(get(param.pb_3,'tag'))*2)+1;
end


%% define the delta differences for the next round; deduce and count turnarounds

switch param.playback(1,param.counter)
    case 1
        % logic to determine DIRECTION (param.turn_observer, param.turn_counter)
        if param.answer{1}(end)==0
            param.turn_observer{1}(ceil(param.counter/3)) =  1; % case -
        else
            if size(param.answer{1},2) >= 2
                if param.answer{1}(end-1)==1
                    if param.turn_observer{1}(end)==0
                        param.turn_observer{1}(ceil(param.counter/3)) = -1; % case ++
                    else
                        param.turn_observer{1}(ceil(param.counter/3)) = 0;  % case (++)+
                    end
                else
                    param.turn_observer{1}(ceil(param.counter/3)) = 0;   % case -+
                end
            else
                param.turn_observer{1}(ceil(param.counter/3)) = 0; % case +
            end
        end
        
        %logic to determine a turnaround point (-1 -1 1, 0 -1 1, 1 0 -1, -1 0 1)
        %are the only combinations of increase(1), stable(0), decrease(-1), that lead to turnarounds
        if param.counter>=7 &&(...
                (param.turn_observer{1}(ceil(param.counter/3)-2)==-1 && param.turn_observer{1}(ceil(param.counter/3)-1)==-1 && param.turn_observer{1}(ceil(param.counter/3))==1)  || ...
                (param.turn_observer{1}(ceil(param.counter/3)-2)== 0 && param.turn_observer{1}(ceil(param.counter/3)-1)==-1 && param.turn_observer{1}(ceil(param.counter/3))==1)  || ...
                (param.turn_observer{1}(ceil(param.counter/3)-2)== 1 && param.turn_observer{1}(ceil(param.counter/3)-1)== 0 && param.turn_observer{1}(ceil(param.counter/3))==-1) || ...
                (param.turn_observer{1}(ceil(param.counter/3)-2)==-1 && param.turn_observer{1}(ceil(param.counter/3)-1)== 0 && param.turn_observer{1}(ceil(param.counter/3))==1) )
            param.turn_counter(1)=param.turn_counter(1)+1;
            if param.turn_counter(1)==1
                param.turn_values{1}=param.dif{1}(end);
            else
                param.turn_values{1}=[param.turn_values{1} param.dif{1}(end)];
            end
        end
        
        % define DELTA for next round (scaler, param.dif)
        if param.turn_counter(1)==1; param.scaler(1)=0.5; end
        if param.turn_counter(1)==2; param.scaler(1)=0.5; end
        if param.turn_counter(1)>=3; param.scaler(1)=0.25; end
        
        switch param.turn_observer{1}(end)
            case -1
                if size(param.dif{1},2)>=4 % logic if stuck
                    if mean(param.dif{1}(end-3:end))<=param.scaler(1)*param.step(1)
                        if param.scaler(1)==0.25
                            param.expert(1)=1;
                        else
                            param.scaler(1)=param.scaler(1)/2;
                        end
                    end
                end
                param.dif{1} = [param.dif{1} max(param.step(1)/2*param.scaler(1),param.dif{1}(end)-param.step(1)*param.scaler(1))];
            case 0
                param.dif{1} = [param.dif{1} param.dif{1}(end)];
            case 1
                param.dif{1} = [param.dif{1} param.dif{1}(end)+param.step(1)*param.scaler(1)];
        end
        
        
    case 2
        % logic to determine DIRECTION (param.turn_observer, param.turn_counter)
        if param.answer{2}(end)==0
            param.turn_observer{2}(ceil(param.counter/3)) =  1; % case -
        else
            if size(param.answer{2},2) >= 2
                if param.answer{2}(end-1)==1
                    if param.turn_observer{2}(end)==0
                        param.turn_observer{2}(ceil(param.counter/3)) = -1; % case ++
                    else
                        param.turn_observer{2}(ceil(param.counter/3)) = 0;  % case (++)+
                    end
                else
                    param.turn_observer{2}(ceil(param.counter/3)) = 0;   % case -+
                end
            else
                param.turn_observer{2}(ceil(param.counter/3)) = 0; % case +
            end
        end
        
        %logic to determine a turnaround point (-1 -1 1, 0 -1 1, 1 0 -1, -1 0 1)
        %are the only combinations of increase(1), stable(0), decrease(-1), that lead to turnarounds
        if param.counter>=7 &&(...
                (param.turn_observer{2}(ceil(param.counter/3)-2)==-1 && param.turn_observer{2}(ceil(param.counter/3)-1)==-1 && param.turn_observer{2}(ceil(param.counter/3))==1)  || ...
                (param.turn_observer{2}(ceil(param.counter/3)-2)== 0 && param.turn_observer{2}(ceil(param.counter/3)-1)==-1 && param.turn_observer{2}(ceil(param.counter/3))==1)  || ...
                (param.turn_observer{2}(ceil(param.counter/3)-2)== 1 && param.turn_observer{2}(ceil(param.counter/3)-1)== 0 && param.turn_observer{2}(ceil(param.counter/3))==-1) || ...
                (param.turn_observer{2}(ceil(param.counter/3)-2)==-1 && param.turn_observer{2}(ceil(param.counter/3)-1)== 0 && param.turn_observer{2}(ceil(param.counter/3))==1) )
            param.turn_counter(2)=param.turn_counter(2)+1;
            if param.turn_counter(2)==1
                param.turn_values{2}=param.dif{2}(end);
            else
                param.turn_values{2}=[param.turn_values{2} param.dif{2}(end)];
            end
        end
        
        % define DELTA for next round (scaler, param.dif)
        if param.turn_counter(2)==1; param.scaler(2)=0.5; end
        if param.turn_counter(2)==2; param.scaler(2)=0.5; end
        if param.turn_counter(2)>=3; param.scaler(2)=0.25; end
        
        switch param.turn_observer{2}(end)
            case -1
                if size(param.dif{2},2)>=4 % logic if stuck
                    if mean(param.dif{2}(end-3:end))<=param.scaler(2)*param.step(2)
                        if param.scaler(2)==0.25
                            param.expert(2)=1;
                        else
                            param.scaler(2)=param.scaler(2)/2;
                        end
                    end
                end
                param.dif{2} = [param.dif{2} max(param.step(2)/2*param.scaler(2),param.dif{2}(end)-param.step(2)*param.scaler(2))];
            case 0
                param.dif{2} = [param.dif{2} param.dif{2}(end)];
            case 1
                param.dif{2} = [param.dif{2} param.dif{2}(end)+param.step(2)*param.scaler(2)];
        end
        
        
    case 3
        % logic to determine DIRECTION (param.turn_observer, param.turn_counter)
        if param.answer{3}(end)==0
            param.turn_observer{3}(ceil(param.counter/3)) =  1; % case -
        else
            if size(param.answer{3},2) >= 2
                if param.answer{3}(end-1)==1
                    if param.turn_observer{3}(end)==0
                        param.turn_observer{3}(ceil(param.counter/3)) = -1; % case ++
                    else
                        param.turn_observer{3}(ceil(param.counter/3)) = 0;  % case (++)+
                    end
                else
                    param.turn_observer{3}(ceil(param.counter/3)) = 0;   % case -+
                end
            else
                param.turn_observer{3}(ceil(param.counter/3)) = 0; % case +
            end
        end
        
        %logic to determine a turnaround point (-1 -1 1, 0 -1 1, 1 0 -1, -1 0 1)
        %are the only combinations of increase(1), stable(0), decrease(-1), that lead to turnarounds
        if param.counter>=7 &&(...
                (param.turn_observer{3}(ceil(param.counter/3)-2)==-1 && param.turn_observer{3}(ceil(param.counter/3)-1)==-1 && param.turn_observer{3}(ceil(param.counter/3))==1)  || ...
                (param.turn_observer{3}(ceil(param.counter/3)-2)== 0 && param.turn_observer{3}(ceil(param.counter/3)-1)==-1 && param.turn_observer{3}(ceil(param.counter/3))==1)  || ...
                (param.turn_observer{3}(ceil(param.counter/3)-2)== 1 && param.turn_observer{3}(ceil(param.counter/3)-1)== 0 && param.turn_observer{3}(ceil(param.counter/3))==-1) || ...
                (param.turn_observer{3}(ceil(param.counter/3)-2)==-1 && param.turn_observer{3}(ceil(param.counter/3)-1)== 0 && param.turn_observer{3}(ceil(param.counter/3))==1) )
            param.turn_counter(3)=param.turn_counter(3)+1;
            if param.turn_counter(3)==1
                param.turn_values{3}=param.dif{3}(end);
            else
                param.turn_values{3}=[param.turn_values{3} param.dif{3}(end)];
            end
        end
        
        % define DELTA for next round (scaler, param.dif)
        if param.turn_counter(3)==1; param.scaler(3)=0.5; end
        if param.turn_counter(3)==2; param.scaler(3)=0.5; end
        if param.turn_counter(3)>=3; param.scaler(3)=0.25; end
        switch param.turn_observer{3}(end)
            case -1
                if size(param.dif{3},2)>=4 % logic if stuck
                    if mean(param.dif{3}(end-3:end))<=param.scaler(3)*param.step(3)
                        if param.scaler(3)==0.25
                            param.expert(3)=1;
                        else
                            param.scaler(3)=param.scaler(3)/2;
                        end
                    end
                end
                param.dif{3} = [param.dif{3} max(param.step(3)/2*param.scaler(3),param.dif{3}(end)-param.step(3)*param.scaler(3))];
            case 0
                param.dif{3} = [param.dif{3} param.dif{3}(end)];
            case 1
                param.dif{3} = [param.dif{3} param.dif{3}(end)+param.step(3)*param.scaler(3)];
        end
end

fprintf('\nNumber of reversals: \n D1 (level):      %d \n D2 (pitch):      %d \n D3 (duration):   %d \n \n', param.turn_counter)
fprintf('Current values: \n D1 (level):      %.2f dB\n D2 (pitch):      %.2f Hz \n D3 (duration):   %.3f sec\n \n',param.dif{1}(end),param.dif{2}(end),param.dif{3}(end))
fprintf('-------------------------------------------------------------------------\n')

%% add new 4-element sequence with the opposite meters for each dimension
%% when the counter is at the end
if mod(param.counter,3) == 0
    add = [randperm(3);zeros(1,3)];
    if param.playback(1,end) == add(1,1)
        add(1,1) = add(1,2);
        add(1,2) = param.playback(1,end);
    end
    for ii=1:3
        loc=find(param.playback(1,end-2:end)==ii);
        met=param.playback(2,end-3+loc); % 1.Position without fffset
        if met==3; add(2,find(add(1,:)==ii))=2;
        else add(2,find(add(1,:)==ii))=3;
        end
    end
    param.playback=[param.playback add];
end

%% calculation and playback of the downbeat of next trial
param.counter=param.counter+1;

if param.playback(1,param.counter)==3
    t = 0:1/param.fs:param.dur+param.dif{3}(end)-1/param.fs;
else t = 0:1/param.fs:param.dur-1/param.fs; end
T = t'*(1:param.Nharm);
if param.playback(1,param.counter)==2
    DOWN = 0.01 * sin(T*2*pi*(param.f0+param.dif{2}(end))+repmat(param.phase,size(T,1),1));
else DOWN = 0.01 * sin(T*2*pi*param.f0+repmat(param.phase,size(T,1),1)); end
harm = ones(1,param.Nharm);
roll = 10^(param.roll/20);
param.roll_factor=1;
for i = 2:param.Nharm
    harm(i) = roll^(i-1)*param.roll_factor;
end
DOWN = DOWN.*repmat(harm,size(t,2),1);
if param.playback(1,param.counter)==1
    down = sum(DOWN,2);
    down = down * 10^((param.calib+param.dif{1}(end))/20);
else down = sum(DOWN,2)*10^((param.calib)/20); end

% fade-in and -out
fade_samples = 1:round(param.fs*param.fade_time);
fade = sin(pi/2*[fade_samples]/fade_samples(end));
down(fade_samples) = down(fade_samples).*fade';
down(end-fade_samples(end)+1:end) = down(end-fade_samples(end)+1:end).*fade(end:-1:1)';

%fill in zeros
param.down=[down; zeros(round(60*param.fs/param.bpm)-size(down,1),1)];

set(hObject,'enable','off')
set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
set(param.pb_3,'BackgroundColor',param.color_pb,'tag','0');
param.time(param.counter-1)=toc;

if (param.turn_counter(1)<param.reversals && param.expert(1)==0) || (param.turn_counter(2)<param.reversals && param.expert(2)==0) || (param.turn_counter(3)<param.reversals && param.expert(3)==0)
    sig=repmat([param.down; repmat(param.up,param.playback(2,param.counter)-1,1)],5-param.playback(2,param.counter),1);
    sig=sig(1:end-param.zeros);
    if max(abs(sig))<0.99
        pause(0.5)
        set(param.pb_2,'BackgroundColor',param.color_pb);
        set(param.pb_3,'BackgroundColor',param.color_pb);
        param.player=audioplayer(sig,param.fs);
        pause(0.001)
        playblocking(param.player);
        pause(0.01)
        set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_3,'BackgroundColor',param.color_pb,'tag','0');
        set(hObject,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
else
    % create folder if it doesn't exist
    help1=dir;
    help2=0;
    for ii=1:size(help1,1)
        if strcmp('results',help1(ii).name)
            help2=help2+1;
        end
    end
    if help2==0;
        mkdir('results')
    end
    
    % plot and save
    if param.expert(1)
        JND{1} = 0.25*param.step(1);
    else
        JND{1} = mean(param.turn_values{1}(end-param.lastrev+1:end));
    end
    if param.expert(2)
        JND{2} = 0.25*param.step(2);
    else
        JND{2} = mean(param.turn_values{2}(end-param.lastrev+1:end));
    end
    if param.expert(3)
        JND{3} = 0.25*param.step(3);
    else
        JND{3} = mean(param.turn_values{3}(end-param.lastrev+1:end));
    end
    figure, plot(param.dif{1}); title('Adaptation curve for D1 (level)'), xlabel('trial'), ylabel('dB')
    saveas(gcf,['results/' param.pathnfile{1} '_' param.pathnfile{2},'_D1'],'fig');
    figure, plot(param.dif{2}); title('Adaptation curve for D2 (frequency)'), xlabel('trial'), ylabel('Hz')
    saveas(gcf,['results/' param.pathnfile{1} '_' param.pathnfile{2},'_D2'],'fig');
    figure, plot(param.dif{3}*1000); title('Adaptation curve for D3 (duration)'), xlabel('trial'), ylabel('ms')
    saveas(gcf,['results/' param.pathnfile{1} '_' param.pathnfile{2},'_D3'],'fig');
    
    METER_test={param.playback, param.answer, param.dif, param.turn_observer, param.turn_counter,...
        param.turn_values, JND, param.counter, param.time, param.AB};
    
    if exist(['results\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'])
        helpi=load(['results\' param.pathnfile{1} '_' param.pathnfile{2} '.mat']);
        if isfield(helpi,'METER_test')
            [filename, pathname]=uiputfile('*.mat','Save results as',['results\' param.pathnfile{1} '_' param.pathnfile{2} '_2.mat']);
            save([pathname '/' filename],'METER_test');
        else
            save(['results\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'],'METER_test','-append');
        end
    else
        save(['results\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'],'METER_test');
    end
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], {'Discrimination thresholds'}, 'Sheet1', 'A1');
    D1_10={'D1 (dB)', 'D2 (Hz)', 'D3 (sec)', 'D4 (Hz)', 'D5 (Hz)', 'D6 (dB)', 'D7 (dB)', 'D8 (dB/harm)', 'D9 (sec)', 'D10 (dB)'};
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], D1_10, 'Sheet1', 'A2:J2');
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], JND{1}, 'Sheet1', 'A3');
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], JND{2}, 'Sheet1', 'B3');
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], JND{3}, 'Sheet1', 'C3');
    
    fprintf('-------------------------------------------------------------------------\n')
    fprintf('Difference thresholds: \n D1 (level):      %.2f dB\n D2 (pitch):      %.2f Hz \n D3 (duration):   %.3f sec\n \n',JND{1},JND{2},JND{3})
    fprintf('-------------------------------------------------------------------------\n')
    close(param.main)
end