 function meter_training(pathnfile,calib,language)

%% Settings 
param.lev_dif =6; %     difference in D1 level (dB) 
param.f0_dif = 15;   %  difference in D2 pitch (Hz)
param.dur_dif = 0.25; % difference in D3 duration (sec) - all differences constant in training phase

%% init
if nargin<3, language=0; end
if nargin<2, calib=0; end
if nargin<1, pathnfile={'testperson','noHA'}; end

param.pathnfile=pathnfile;
param.calib=calib;
param.language = language;

% param.answer: Answers for all trials, 1 right, 0 wrong
% param.playback: 1st row = the dimension under test (1:lev, 2:f0,
% 3:dur); 2nd row = the played back meter of that trial
% param.lev_dif, ..f0_dif, ..dur_dif: show the history of the
% differences that have been under test for the particular dimension

scsze = get(0,'ScreenSize');
height = 500;
length = 510;
pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];

param.main = figure('Name','Meter: Training','NumberTitle','off');
param.color=get(param.main,'Color');
param.color_active = [0.54 0.7 1]; % here no color_activepush necessary
param.color_right = [0.4 0.7 0.2];
param.color_wrong = [1 0.2 0.2];
set(param.main,'MenuBar','none');
set(param.main,'Position', pos);
param.caption = uicontrol('Style', 'text', 'String', 'Meter: Training', 'FontSize', 14, 'FontWeight', 'bold', 'Position', [length/2 - 180 height-60 360 30],'BackgroundColor',param.color);


param.txt1 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'After pressing "Start", you will hear a sequence of beats.', 'Position', [length/2-210 height-120 420 30], 'BackgroundColor',param.color);
param.txt2 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'Please decide whether it is a 2/4 or 3/4 meter. Confirm', 'Position', [length/2-210 height-150 420 30], 'BackgroundColor',param.color);
param.txt3 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'your selection by pressing "Continue". The correct', 'Position', [length/2-210 height-180 420 30], 'BackgroundColor',param.color);
param.txt4 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'answer will be highlighted in green.', 'Position', [length/2-210 height-210 420 30], 'BackgroundColor',param.color);


if param.language
    set(param.main,'Name','Metrum: Training');
    set(param.caption,'String','Metrum: Training');
    set(param.txt1,'String','Nachdem Sie "Start" drcken, hren Sie eine Tonfolge.');
    set(param.txt2,'String','Entscheiden Sie, ob Sie die Tne im 2/4 oder 3/4 Takt');
    set(param.txt3,'String','wahrnehmen. Besttigen Sie Ihre Antwort mit "Weiter".')
    set(param.txt4,'String','Die richtige Antwort wird grn markiert, eine falsche rot.')
end

param.pb_multi=uicontrol('Style','pushbutton','String','Start','tag','1','FontSize', 14, 'FontAngle', 'italic', 'FontWeight', 'bold','Position',[length/2-160 height-460 320 60],'Callback',@function_multi);

note=iconread('images/note.png','png');

sum_note=sum(note,3);
for i = 1:size(note,1)
    for ii=1:size(note,2)
       if sum_note(i,ii)>2.5
           note(i,ii,1:3)=NaN;
       end
    end
end


param.pb_2=uicontrol('Style','pushbutton','String','2/4    ', 'cdata',note, 'tag','0', 'FontSize', 24, 'Position',[length/2-160 height-360 120 120],'Callback',@function_pb);
param.pb_3=uicontrol('Style','pushbutton','String','3/4    ', 'cdata',note, 'tag','0', 'FontSize', 24, 'Position',[length/2+40 height-360 120 120],'Callback',@function_pb);
param.color_pb=get(param.pb_3,'BackgroundColor');

%% stimulus design
param.fs=44100;
param.bpm = 100;
param.dur = 0.28;
param.f0 = 220;
param.Nharm = 40;
param.roll = -2;
param.fade_time=0.08;


% phase
param.Nharm = 40;
t = 0:1/param.fs:param.dur-1/param.fs;
T = t'*(1:param.Nharm);

%% calculation of the standard upbeat
load('randparam.mat');
param.phase=randparam.phase(1:param.Nharm);
UP = 0.01 * sin(T*2*pi*param.f0+repmat(param.phase,size(T,1),1));
harm = ones(1,param.Nharm);
roll = 10^(param.roll/20);
for i = 2:param.Nharm
    harm(i) = roll^(i-1);
end
UP = UP.*repmat(harm,size(t,2),1);
up = sum(UP,2);
up = up * 10^(param.calib/20);

% fade-in and -out
fade_samples = 1:round(param.fs*param.fade_time);
fade = sin(pi/2*[fade_samples]/fade_samples(end));
up(fade_samples) = up(fade_samples).*fade';
up(end-fade_samples(end)+1:end) = up(end-fade_samples(end)+1:end).*fade(end:-1:1)';

%fill in zeros
param.up=[up; zeros(round(60*param.fs/param.bpm)-size(up,1),1)];

param.counter=1;
param.playback=[[2 2 3];...
    round(randperm(3)/1.5)+1];
param.answer=0;



%% preparation of downbeat for first round
if param.playback(1,param.counter)==3
    t = 0:1/param.fs:param.dur+param.dur_dif(end)-1/param.fs;
end
T = t'*(1:param.Nharm);
if param.playback(1,param.counter)==2
    DOWN = 0.01 * sin(T*2*pi*(param.f0+param.f0_dif(end))+repmat(param.phase,size(T,1),1));
else DOWN = 0.01 * sin(T*2*pi*param.f0+repmat(param.phase,size(T,1),1)); end
harm = ones(1,param.Nharm);
roll = 10^(param.roll/20);
for i = 2:param.Nharm
    harm(i) = roll^(i-1);
end
DOWN = DOWN.*repmat(harm,size(t,2),1);
if param.playback(1,param.counter)==1
    down = sum(DOWN,2);
    down = down * 10^((param.calib+param.lev_dif(end))/20);
else down = sum(DOWN,2)*10^((param.calib)/20); end

% fade-in and -out
fade_samples = 1:round(param.fs*param.fade_time);
fade = sin(pi/2*[fade_samples]/fade_samples(end));
down(fade_samples) = down(fade_samples).*fade';
down(end-fade_samples(end)+1:end) = down(end-fade_samples(end)+1:end).*fade(end:-1:1)';

%fill in zeros
param.down=[down; zeros(round(60*param.fs/param.bpm)-size(down,1),1)];
param.zeros=round(60*param.fs/param.bpm)-size(down,1);

%% preparation, param.playback contains order of dimensions (1= level,
%% 2=pitch, 3=duration, 4=roll off) and the played back meter (2 or 3)

guidata(param.main,param)


%%%%-------------------GUI functions-----------------%%%%
function function_pb(hObject,eventdata)
param=guidata(hObject);
set(param.pb_2,'tag','0','BackgroundColor',param.color_pb);
set(param.pb_3,'tag','0','BackgroundColor',param.color_pb);
set(hObject,'tag','1','BackgroundColor',param.color_active);
guidata(param.main,param);



function function_multi(hObject,eventdata)
param=guidata(hObject);

% only first time to change the String
if strcmp(get(hObject,'String'),'Start')
    tic;
    set(hObject,'String','Continue','enable','off')
    if param.language
        set(hObject,'String','Weiter')
    end
    set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
    set(param.pb_3,'BackgroundColor',param.color_pb,'tag','0');
    sig=repmat([param.down; repmat(param.up,param.playback(2,param.counter)-1,1)],5-param.playback(2,param.counter),1);
    if max(abs(sig))<0.99
        pause(0.5)
        set(param.pb_2,'BackgroundColor',param.color_pb);
        set(param.pb_3,'BackgroundColor',param.color_pb);
        param.player=audioplayer(sig,param.fs);
        pause(0.001)
        playblocking(param.player);
        pause(0.01)
        set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_3,'BackgroundColor',param.color_pb,'tag','0');
        set(hObject,'enable','on')
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
    return
end
stop(param.player)

%% return if no answer is selected ( not needed as "Confirm button" is
%% desactivated

if strcmp(get(param.pb_2,'tag'),get(param.pb_3,'tag')) % both 0
    return
end

set(hObject,'enable','off')
% define whether the answer is right(1) or wrong(0)
if str2num(get(param.pb_2,'tag'))*2+str2num(get(param.pb_3,'tag'))*3==param.playback(2,param.counter)
    param.answer(param.counter)=1;
    set(eval(['param.pb_' num2str(param.playback(2,param.counter))]),'BackgroundColor',param.color_right);
else
    param.answer(param.counter)=0;
    set(eval(['param.pb_' num2str(5-param.playback(2,param.counter))]),'BackgroundColor',param.color_wrong);
end


%% add new 4-element sequence with the opposite meters for each dimension
%% when the counter is at the end
if mod(param.counter,3) == 0
    add = [randperm(3);zeros(1,3)];
    if param.playback(1,end) == add(1,1)
        add(1,1) = add(1,2);
        add(1,2) = param.playback(1,end);
    end
    for ii=1:3
        loc=find(param.playback(1,end-2:end)==ii);
        met=param.playback(2,end-3+loc); % 1.Stelle ohne Offset
        if met==3; add(2,find(add(1,:)==ii))=2;
        else add(2,find(add(1,:)==ii))=3;
        end
    end
    param.playback=[param.playback add];
end

%% calculation and playback of the downbeat of next trial
param.counter=param.counter+1;

if param.playback(1,param.counter)==3
    t = 0:1/param.fs:param.dur+param.dur_dif(end)-1/param.fs;
else t = 0:1/param.fs:param.dur-1/param.fs; end
T = t'*(1:param.Nharm);
if param.playback(1,param.counter)==2
    DOWN = 0.01 * sin(T*2*pi*(param.f0+param.f0_dif(end))+repmat(param.phase,size(T,1),1));
else DOWN = 0.01 * sin(T*2*pi*param.f0+repmat(param.phase,size(T,1),1)); end
harm = ones(1,param.Nharm);
roll = 10^(param.roll/20);
param.roll_factor=1;
for i = 2:param.Nharm
    harm(i) = roll^(i-1)*param.roll_factor;
end
DOWN = DOWN.*repmat(harm,size(t,2),1);
if param.playback(1,param.counter)==1
    down = sum(DOWN,2);
    down = down * 10^((param.calib+param.lev_dif(end))/20);
else down = sum(DOWN,2)*10^((param.calib)/20); end

% fade-in and -out
fade_samples = 1:round(param.fs*param.fade_time);
fade = sin(pi/2*[fade_samples]/fade_samples(end));
down(fade_samples) = down(fade_samples).*fade';
down(end-fade_samples(end)+1:end) = down(end-fade_samples(end)+1:end).*fade(end:-1:1)';

%fill in zeros
param.down=[down; zeros(round(60*param.fs/param.bpm)-size(down,1),1)];

pause(1.5) % color pause
set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
set(param.pb_3,'BackgroundColor',param.color_pb,'tag','0');
param.time(param.counter-1)=toc;
if param.counter<=6
    sig=repmat([param.down; repmat(param.up,param.playback(2,param.counter)-1,1)],5-param.playback(2,param.counter),1);
    sig=sig(1:end-param.zeros);
    if max(abs(sig))<0.99
        pause(0.5)
        set(param.pb_2,'BackgroundColor',param.color_pb);
        set(param.pb_3,'BackgroundColor',param.color_pb);
        param.player=audioplayer(sig,param.fs);
        pause(0.001)
        playblocking(param.player);
        pause(0.01)
        set(param.pb_2,'BackgroundColor',param.color_pb,'tag','0');
        set(param.pb_3,'BackgroundColor',param.color_pb,'tag','0');
        set(hObject,'enable','on')
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
else
    % create folder if it doesn't exist
    help1=dir;
    help2=0;
    for ii=1:size(help1,1)
        if strcmp('results',help1(ii).name)
            help2=help2+1;
        end
    end
    if help2==0;
        mkdir('results')
    end
    
    close(param.main)
    meter(param.pathnfile, param.calib, param.language)
end