function music_calibration(pathnfile, language)

%% Settings
param.dif = -30; % initial level (dB)
param.step = 4; % step size (dB)
param.reversals = 6; % number or reversals that are needed to end the adaptation
param.lastrev = 4; % number of reversals that are used to calculate the difference threshold

%% init
if nargin<2, language=0; end
if nargin<1, pathnfile={'test_person','noHA'}; end

%% window design
scsze = get(0,'ScreenSize');

param.pathnfile=pathnfile;
param.language = language;
height = 500;
length = 610;
pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];

param.main = figure('Name','Music Calibration','NumberTitle','off');
param.color=get(param.main,'Color');
param.color_active = [0.54 0.7 1];
param.color_activepush = [0.541 0.7 1];
param.color_right = [0.4 0.7 0.2];
param.color_wrong = [1 0.2 0.2];
set(param.main,'MenuBar','none');
set(param.main,'Position', pos);
param.caption = uicontrol('Style', 'text', 'String', 'Music Calibration', 'FontSize', 14, 'FontWeight', 'bold', 'Position', [length/2 - 180 height-60 360 30],'BackgroundColor',param.color);

param.txt1 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'After pressing "Start", the buttons "1" and "2" are highlighted one', 'Position', [length/2-250 height-120 500 30], 'BackgroundColor',param.color);
param.txt2 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'after the other. Only one of the buttons plays back an audio', 'Position', [length/2-250 height-150 500 30], 'BackgroundColor',param.color);
param.txt3 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'signal. Select the respective button and confirm your answer', 'Position', [length/2-250 height-180 500 30], 'BackgroundColor',param.color);
param.txt4 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'by pressing "Continue".', 'Position', [length/2-250 height-210 500 30], 'BackgroundColor',param.color);

param.pb_multi=uicontrol('Style','pushbutton','String','Start','tag','1','FontSize', 14, 'FontAngle', 'italic', 'FontWeight', 'bold','Position',[length/2-180 height-460 360 60],'Callback',@function_multi);

param.pb_1=uicontrol('Style','pushbutton','String','1', 'tag', '0', 'FontSize', 20, 'Position',[length/2-180 height-360 120 120],'Callback',@function_pb);
param.pb_2=uicontrol('Style','pushbutton','String','2', 'tag', '0', 'FontSize', 20, 'Position',[length/2+60 height-360 120 120],'Callback',@function_pb);
param.color_pb=get(param.pb_1,'BackgroundColor');

if param.language
    set(param.main,'Name','Test Kalibrierung');
    set(param.caption,'String','Test Kalibrierung');
    set(param.txt1,'String','Nachdem Sie "Start" drcken, leuchten die Knpfe "1" und "2"')
    set(param.txt2,'String','hintereinander auf. Bei nur einem der zwei Knpfe wird whrend')
    set(param.txt3,'String','des Aufleuchtens auch ein Audiosignal abgespielt. Whlen Sie')
    set(param.txt4,'String','diesen Knopf aus und besttigen Sie Ihre Auswahl mit "Weiter".')
end

%% create audio signal
param.fs = 44100;
param.bpm = 100;
param.dur = 1.8;
param.f0 = 220;
param.Nharm = 40;
param.roll = -2;
param.fade_time = 0.04;
load('randparam.mat');
param.phase=randparam.phase(1:param.Nharm);

t = 0:1/param.fs:param.dur-1/param.fs;
T = t'*(1:param.Nharm);
SIG = 0.01 * sin(T*2*pi*param.f0+repmat(param.phase,size(T,1),1));
harm = ones(1,param.Nharm);
roll = 10^(param.roll/20);
for i = 2:param.Nharm
    harm(i) = roll^(i-1);
end
SIG = SIG.*repmat(harm,size(t,2),1);
sig = sum(SIG,2);

% fade-in and -out
fade_samples = 1:round(param.fs*param.fade_time);
fade = sin(pi/2*[fade_samples]/fade_samples(end));
sig(fade_samples) = sig(fade_samples).*fade';
sig(end-fade_samples(end)+1:end) = sig(end-fade_samples(end)+1:end).*fade(end:-1:1)';
param.sig = sig;
param.player=audioplayer(0,param.fs);
guidata(param.main,param)

%% adaptive process
param.counter=1;
param.playback = ceil(randperm(4)/2);
param.answer=0;
param.turn_observer=0;
param.turn_counter=0;
param.turn_values=0;
param.timehelper=0;
param.scaler=1;
guidata(param.main,param);

%%------------------- GUI functions ----------------------%%%%
function function_pb(hObject,eventdata)
param=guidata(hObject);
if ~(sum(get(param.pb_1,'BackgroundColor')==param.color_active)==3 || sum(get(param.pb_2,'BackgroundColor')==param.color_active)==3)
    set(param.pb_1,'tag','0','BackgroundColor',param.color_pb);
    set(param.pb_2,'tag','0','BackgroundColor',param.color_pb);
    set(hObject,'tag','1','BackgroundColor',param.color_activepush);
    guidata(param.main,param);
end


function function_multi(hObject,eventdata)
param=guidata(hObject);

if strcmp(get(hObject,'String'),'Start')
    set(param.pb_multi,'enable','off');
    if param.timehelper==0
        tic; param.timehelper=1;
    end
    set(param.pb_1,'tag','0','BackgroundColor',param.color_pb);
    set(param.pb_2,'tag','0','BackgroundColor',param.color_pb);
    stop(param.player)
    sig=param.sig*10^(param.dif(param.counter)/20);
    active=audioplayer(sig,param.fs);
    mute=audioplayer(sig*0,param.fs);
    pause(0.5)
    set(param.pb_2,'BackgroundColor',param.color_pb);
    set(param.pb_1,'BackgroundColor',param.color_active);
    if param.playback(param.counter)==1
        param.player=active;
    else
        param.player=mute;
    end
    pause(0.001)
    playblocking(param.player)
    pause(0.001)
    set(param.pb_1,'BackgroundColor',param.color_pb);
    set(param.pb_2,'BackgroundColor',param.color_pb);
    pause(0.5)
    set(param.pb_1,'BackgroundColor',param.color_pb);
    set(param.pb_2,'BackgroundColor',param.color_active);
    if param.playback(param.counter)==2
        param.player=active;
    else
        param.player=mute;
    end
    pause(0.001)
    playblocking(param.player)
    pause(0.001)
    set(param.pb_1,'BackgroundColor',param.color_pb);
    set(param.pb_2,'BackgroundColor',param.color_pb);
    set(param.pb_multi,'String','Continue','enable','on');
    if param.language
        set(param.pb_multi,'String','Weiter')
    end
    guidata(param.main,param);
    return
end

% return if nothing is chosen
if str2num(get(param.pb_1,'tag'))==0 && str2num(get(param.pb_2,'tag'))==0
    return
end
set(param.pb_multi,'enable','off');

%% answer this round
param.time(param.counter) = toc;
if param.turn_counter<param.reversals
    if str2num(get(param.pb_1,'tag'))*1 + str2num(get(param.pb_2,'tag'))*2 == param.playback(param.counter)
        param.answer(param.counter)=1;
    else
        param.answer(param.counter)=0;
    end
    %% define delta for next round
    
    if param.answer(end)==0
        param.turn_observer(param.counter) =  1; % case -
    else
        if size(param.answer,2) >= 2
            if param.answer(end-1)==1
                if param.turn_observer(end)==0
                    param.turn_observer(param.counter) = -1; % case ++
                else
                    param.turn_observer(param.counter) = 0;  % case (++)+
                end
            else
                param.turn_observer(param.counter) = 0;   % case -+
            end
        else
            param.turn_observer(param.counter) = 0; % case +
        end
    end
    
    
    %logic to determine a turnaround point (-1 -1 1, 0 -1 1, 1 0 -1, -1 0 1)
    %are the only combinations of increase(1), stable(0), decrease(-1), that lead to turnarounds
    if param.counter>=3 &&(...
            (param.turn_observer(param.counter-2)==-1 && param.turn_observer(param.counter-1)==-1 && param.turn_observer(param.counter)==1)  || ...
            (param.turn_observer(param.counter-2)== 0 && param.turn_observer(param.counter-1)==-1 && param.turn_observer(param.counter)==1)  || ...
            (param.turn_observer(param.counter-2)== 1 && param.turn_observer(param.counter-1)== 0 && param.turn_observer(param.counter)==-1) || ...
            (param.turn_observer(param.counter-2)==-1 && param.turn_observer(param.counter-1)== 0 && param.turn_observer(param.counter)==1) )
        param.turn_counter=param.turn_counter+1;
        if param.turn_counter==1
            param.turn_values=param.dif(end);
        else
            param.turn_values=[param.turn_values param.dif(end)];
        end
    end
    
    
    % define DELTA for next round (scaler, param.dif)
    if param.turn_counter==1; param.scaler=0.5; end
    if param.turn_counter>=2; param.scaler=0.25; end
    
    switch param.turn_observer(end)
        case -1
            param.dif = [param.dif param.dif(end)-param.step*param.scaler];
        case 0
            param.dif = [param.dif param.dif(end)];
        case 1
            param.dif = [param.dif param.dif(end)+param.step*param.scaler];
    end
    
    
    
    %% add new 4-element playback sequence when the counter is at the end
    if mod(param.counter,4) == 0
        param.playback=[param.playback ceil(randperm(4)/2)];
    end
    
    fprintf('\nNumber of reversals:     %d \n', param.turn_counter)
    fprintf('Current value:         %.0f dB\n \n',param.dif(end))
    fprintf('-------------------------------------------------------------------------\n')
    %% prepare next round
    param.counter = param.counter+1;
    set(param.pb_1,'tag','0','BackgroundColor',param.color_pb);
    set(param.pb_2,'tag','0','BackgroundColor',param.color_pb);
    stop(param.player)
    set(param.pb_multi,'enable','off');
    sig = param.sig*10^(param.dif(param.counter)/20);
    active = audioplayer(sig,param.fs);
    mute = audioplayer(sig*0,param.fs);
    pause(0.5)
    set(param.pb_2,'BackgroundColor',param.color_pb);
    set(param.pb_1,'BackgroundColor',param.color_active);
    if param.playback(param.counter) == 1
        param.player = active;
    else
        param.player = mute;
    end
    pause(0.001)
    playblocking(param.player)
    set(param.pb_1,'BackgroundColor',param.color_pb);
    set(param.pb_2,'BackgroundColor',param.color_pb);
    pause(0.5)
    set(param.pb_1,'BackgroundColor',param.color_pb);
    set(param.pb_2,'BackgroundColor',param.color_active);
    if param.playback(param.counter)==2
        param.player=active;
    else
        param.player=mute;
    end
    pause(0.001)
    playblocking(param.player)
    set(param.pb_1,'BackgroundColor',param.color_pb);
    set(param.pb_2,'BackgroundColor',param.color_pb);
    set(param.pb_multi,'enable','on')
    
    guidata(param.main,param);
else
    stop(param.player)
    calib = mean(param.turn_values(1,end-param.lastrev+1:end)); % in dB
    MUSIC_calib={param.playback, param.answer, param.dif, param.turn_observer, param.turn_counter,...
        param.turn_values, calib, param.counter, param.time};
    
    % create folder if it doesn't exist
    help1=dir;
    help2=0;
    for ii=1:size(help1,1)
        if strcmp('calib',help1(ii).name)
            help2=help2+1;
        end
    end
    if help2==0;
        mkdir('calib')
    end
    figure, plot(param.dif); title('Adaptation for treshold detection'), xlabel('trial'), ylabel('dB')
    saveas(gcf,['results/' param.pathnfile{1} '_' param.pathnfile{2},'_calib'],'fig');
    if exist(['calib\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'])
        helpi=load(['calib\' param.pathnfile{1} '_' param.pathnfile{2} '.mat']);
        if isfield(helpi,'MUSIC_calib')
            if ~isfield(helpi,'timbre_calib')
                [filename, pathname]=uiputfile('*.mat','Save results as',['results\' param.pathnfile{1} '_' param.pathnfile{2} '_2.mat']);
                save([pathname  filename],'MUSIC_calib');
            else
                [filename, pathname]=uiputfile('*.mat','Save results as',['results\' param.pathnfile{1} '_' param.pathnfile{2} '_2.mat']);
                save([pathname  filename],'MUSIC_calib','-append');
            end
        else
            save(['calib\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'],'MUSIC_calib','-append');
        end
    else
        save(['calib\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'],'MUSIC_calib');
    end
    
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], {'Calibration (dB)'}, 'Sheet1', 'A5');
    xlswrite(['results\' param.pathnfile{1} '_' param.pathnfile{2}], calib, 'Sheet1', 'A6');
    
    fprintf('-------------------------------------------------------------------------\n')
    fprintf('Threshold:  %.1f dB\n',calib)
    fprintf('-------------------------------------------------------------------------\n')
    
    close(param.main)
end