function timbre(pathnfile, calib, language)

%% init
if nargin<3, language=0; end    
if nargin<2, calib=0; end
if nargin<1, pathnfile = {'testperson','noHA'}; end

param.pathnfile = pathnfile;
param.calib = calib;
param.language = language;

%% window design
scsze = get(0,'ScreenSize');
color = [0.831 0.816 0.784];
param.warncolor=[1 0.7 0.3];

height=350;
length=320;
param.height=height;
param.length=length;

pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];

param.main = figure('Name','Timbre','NumberTitle','off');
set(param.main,'MenuBar','none');
set(param.main,'Color', color);
set(param.main,'Position', pos);

param.caption = uicontrol('Style', 'text', 'String', 'Timbre', 'FontSize', 12, 'FontWeight', 'bold', 'Position', [length/2 - 200 height-80 400 40], 'BackgroundColor',color);
param.calibration = uicontrol('Style', 'pushbutton', 'String', 'Calibration', 'FontSize', 10, 'Position', [length/2-70 height-150 140 50], 'BackgroundColor',color,'Callback',@function_calibration);
param.training = uicontrol('Style', 'pushbutton', 'String', 'Training', 'FontSize', 10, 'Position', [length/2-70 height-225 140 50], 'BackgroundColor',color,'Callback',@function_training);
param.test = uicontrol('Style', 'pushbutton', 'String', 'Test', 'FontSize', 10, 'Position', [length/2-70 height-300 140 50], 'BackgroundColor',color,'Callback',@function_test);

if param.language
    set(param.calibration,'String','Kalibrierung')
end

guidata(param.main,param);

%%%%------------------------GUI function-----------------------%%%%
function function_calibration(hObject, eventdata)
param = guidata(hObject);
timbre_calibration(param.pathnfile,param.calib, param.language);

function function_training(hObject, eventdata)
param = guidata(hObject);
pathnfile=param.pathnfile;
if exist(['calib\' pathnfile{1} '_' pathnfile{2} '.mat'])
    helpi=load(['calib\' pathnfile{1} '_' pathnfile{2} '.mat']);
    if isfield(helpi,'timbre_calib')
%        timbre_training(pathnfile,helpi.MUSIC_calib{7}+40,param.language, helpi.timbre_calib);    % add SL and loudness equalization (calculated with Chalupper loudness model)
        timbre_training(pathnfile,param.calib,param.language, helpi.timbre_calib);
    else
        set(param.calibration,'BackgroundColor',param.warncolor)
        disp(['No previous calibration data for subject ' pathnfile{1} ' & ' pathnfile{2} '. Please calibrate!'])
    end
else
        set(param.calibration,'BackgroundColor',param.warncolor)
        disp(['No previous calibration data for subject ' pathnfile{1} ' & ' pathnfile{2} '. Please calibrate!'])
end

function function_test(hObject, eventdata)
param = guidata(hObject);
pathnfile=param.pathnfile;
if exist(['calib\' pathnfile{1} '_' pathnfile{2} '.mat'])
    helpi=load(['calib\' pathnfile{1} '_' pathnfile{2} '.mat']);
    if isfield(helpi,'timbre_calib')
        % timbre_test(pathnfile,helpi.MUSIC_calib{7}+40,param.language, helpi.timbre_calib);    % add SL and loudness equalization (calculated with Chalupper loudness model)
close(param.main);        
timbre_test(pathnfile,param.calib,param.language, helpi.timbre_calib);
    else
        set(param.calibration,'BackgroundColor',param.warncolor)
        disp(['No previous calibration data for subject ' pathnfile{1} ' & ' pathnfile{2} '. Please calibrate!'])
    end
else
        set(param.calibration,'BackgroundColor',param.warncolor)
        disp(['No previous calibration data for subject ' pathnfile{1} ' & ' pathnfile{2} '. Please calibrate!'])
end