function timbre_calibration(pathnfile, calib, language)

% explanation: param.ref is the reference with -2 roll-off and 0 irr
% param.sig contains 2x param.sig_roll and 2x param.sig_irr, the signals to
% compare and calibrate/equalize loudness

% param.roll defines the default roll-off of reference(1) and test tones
% param.irr defines default irregularity of reference(1) and test tones

%% Setting
param.calib_points = [-0.1 -0.5 -0.9 , -1 -4.25 -7.5 ; 0.45 2.04 3.4 , 0.41 1.42 2.02];   % 1,1:2 roll_off factor, 1,3:4 irr-factor, 2: compensation preamp
% y-values calculated by Fastl/Chalupper model (70dB SPL)

%% init
if nargin<3, language=0; end
if nargin<2, calib=0; end
if nargin<1, pathnfile={'testperson','noHA'}; end

param.pathnfile=pathnfile;
param.calib=calib;
param.language=language;

%% window design
scsze = get(0,'ScreenSize');
height = 540;
length = 620;
pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];
param.pos=pos;

param.main = figure('Name','Timbre: Calibration','NumberTitle','off');
param.color=get(param.main,'Color');
param.color2=[0.4 0.7 0.2];
set(param.main,'MenuBar','none');
set(param.main,'Position', pos);
param.caption = uicontrol('Style', 'text', 'String', 'Timbre: Calibration', 'FontSize', 14, 'FontWeight', 'bold', 'Position', [length/2 - 180 height-90 360 30],'BackgroundColor',param.color);

param.txt1 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'Press "Playback (1/6)" to listen to two different tones. Adjust the level of', 'Position', [length/2-260 height-150 540 30], 'BackgroundColor',param.color);
param.txt2 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'the second stimulus with the slider and press playback again until both', 'Position', [length/2-260 height-180 540 30], 'BackgroundColor',param.color);
param.txt3 = uicontrol('Style', 'text', 'FontSize', 13, 'HorizontalAlignment','center','String', 'stimuli are equally loud. Press "Continue" to confirm your adjustments.', 'Position', [length/2-260 height-210 540 30], 'BackgroundColor',param.color);

param.pb_playback=uicontrol('Style','pushbutton','String','Playback','tag','1','FontSize', 16, 'FontAngle', 'italic', 'FontWeight', 'bold','Position',[length/2-100 height-310 200 70],'Callback',@function_playback);

param.slider=uicontrol('Style','slider','SliderStep',[0.1 0.15],'Min',-3,'Max',+3, 'Value', 0, 'Position',[length/2-250 height-410 500 60],'Callback',@function_slider);
param.text5=uicontrol('Style', 'text', 'FontSize', 40, 'HorizontalAlignment','center','String', '-', 'Position', [length/2-300 height-390 50 50], 'BackgroundColor',param.color);
param.text6=uicontrol('Style', 'text', 'FontSize', 30, 'HorizontalAlignment','center','String', '+', 'Position', [length/2+250 height-400 50 50], 'BackgroundColor',param.color);
param.continue=uicontrol('Style','pushbutton','String','Continue', 'FontSize', 20, 'Position',[length/2+110 height-500 140 60],'Callback',@function_continue);
param.text7=uicontrol('Style', 'text', 'FontSize', 16, 'FontAngle', 'italic', 'FontWeight', 'bold','Position',[length/2+120 height-330 100 70], 'HorizontalAlignment','left','String', '(1/6)', 'BackgroundColor',param.color);

if param.language
    set(param.main,'Name','Timbre: Kalibrierung');
    set(param.caption,'String','Timbre: Kalibrierung');
    set(param.txt1,'String','Drcken Sie "Wiedergabe", um zwei Stimuli zu hren. Verndern Sie die');
    set(param.txt2,'String','Lautstrke des zweiten Stimulus und spielen Sie alles erneut ab, bis beide');
    set(param.txt3,'String','Stimuli gleich laut sind. Besttigen Sie Ihre Einstellung mit "Weiter".');
    set(param.pb_playback,'String','Wiedergabe');
    set(param.continue,'String','Weiter');
end

%% stimulus design
param.fs=44100;
param.dur = 1;
param.f0 = 220;
param.centroid=2000;
param.Nharm = 40;
param.attack_0 = 0.04;
param.roll = -2;
param.fadeout_0=0.04;
attack_samples = 1:round(param.fs*param.attack_0);
attack = (attack_samples/attack_samples(end));
param.fadeout_0=0.04;
fadeout_samples = 1:round(param.fs*param.fadeout_0);
fadeout = (fadeout_samples/fadeout_samples(end));
t = 0:1/param.fs:param.dur-1/param.fs;
T = t'*(1:param.Nharm);

load('randparam.mat')
param.phase=repmat(randparam.phase(1:param.Nharm),size(T,1),1);

% calculation of the standard (reference) tone
REF = 0.01 * sin(2*pi*param.f0*T+param.phase);
harm = ones(1,param.Nharm);
for i = 2:param.Nharm
    harm(i) = (10^(param.roll/20))^(i-1);
end
REF = REF.*repmat(harm,size(T,1),1).*(ones(size(T)));
ref = sum(REF,2);
% fade-in and -out
ref(attack_samples) = ref(attack_samples).*attack';
% standard fade out
ref(end-fadeout_samples(end)+1:end) = ref(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
param.ref = [ref;zeros(round(param.fs*0.4),1)];

% calculation of the "roll" tones
for ii=1:3
SIG = 0.01 * sin(2*pi*param.f0*T+param.phase);
harm = ones(1,param.Nharm);
for i = 2:param.Nharm
    harm(i) = (10^((param.roll+param.calib_points(1,ii))/20))^(i-1);
end
SIG = SIG.*repmat(harm,size(T,1),1).*(ones(size(T)));
sig = sum(SIG,2);
% fade-in and -out
sig(attack_samples) = sig(attack_samples).*attack';
% standard fade out
sig(end-fadeout_samples(end)+1:end) = sig(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
param.sig(:,ii) = [sig;zeros(round(param.fs*0.4),1)];
end

% calculation of the "irr" tones
for ii=4:6
SIG = 0.01 * sin(2*pi*param.f0*T + param.phase);
harm = ones(1,param.Nharm);
for i = 2:param.Nharm
    harm(i) = (10^(param.roll/20))^(i-1);
end
for i = 1:floor(param.Nharm/2)
    harm(2*i)=harm(2*i)*10^(param.calib_points(1,ii)/20);
end
SIG = SIG.*repmat(harm,size(T,1),1).*(ones(size(T)));
sig = sum(SIG,2);
% fade-in and -out
sig(attack_samples) = sig(attack_samples).*attack';
% standard fade out
sig(end-fadeout_samples(end)+1:end) = sig(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
param.sig(:,ii) = [sig;zeros(round(param.fs*0.4),1)];
end

% playback order
param.counter = 1;
param.order = [ 2 4 6 3 5 1; ...   % 1,2: roll with first and second factor ; 3,4: irr...
                2 1 2 2 1 2];        % 1: ref, sig  2: sig, ref
param.calib_data = zeros(1,6);
param.player = audioplayer(0,param.fs);
guidata(param.main,param)

%%%%--------------------------GUI functions-----------------------%%%%
function function_playback(hObject,eventdata)
param=guidata(hObject);
stop(param.player);

help=param.sig(:,param.order(1,param.counter))*10^(param.calib_points(2,param.order(1,param.counter))/20);
param.slider_value=get(param.slider,'Value');

switch param.order(2,param.counter)
    case 1
sig=[param.ref; (10^(param.slider_value/20))*help];
    case 2
sig=[help; param.ref*(10^(param.slider_value/20))];
end

if max(abs(sig))<0.3;
param.player=audioplayer(10^(param.calib/20)*sig,param.fs);
play(param.player);
else
disp('Warning: Signal too loud and not played back for safety reasons!')
end
guidata(param.main,param)

function function_slider(hObject,eventdata)
param=guidata(hObject);

function function_continue(hObject,eventdata)
param=guidata(hObject);
value=get(param.slider,'Value');
if param.order(2,param.counter)==2  % because the order of 'ref' and 'sig' is reversed
    value=-value;
end
param.calib_data(ceil(param.counter/6),param.order(1,param.counter))=value;  % only slider values

if param.counter<size(param.order,2)
    param.counter=param.counter+1;
    set(param.slider,'Value',0);
    if param.language
    %set(param.pb_playback,'String',['Wiedergabe (' num2str(param.counter) '/12)']);
    set(param.text7,'String',['(' num2str(param.counter) '/6)']);
    else
    %set(param.pb_playback,'String',['Playback (' num2str(param.counter) '/12)']);
    set(param.text7,'String',['(' num2str(param.counter) '/6)']);
    end
    guidata(param.main,param)
else
    timbre_calib = [param.calib_points(1,:); param.calib_data+param.calib_points(2,:)];  % add preamps
    timbre_calib_responses = param.calib_data; 
    
    % create folder if it doesn't exist
    help1=dir;
    help2=0;
    for ii=1:size(help1,1)
        if strcmp('calib',help1(ii).name)
            help2=help2+1;
        end
    end
    if help2==0;
        mkdir('calib')
    end
    
     if exist(['calib\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'])
        helpi=load(['calib\' param.pathnfile{1} '_' param.pathnfile{2} '.mat']);
        if isfield(helpi,'timbre_calib')
if ~isfield(helpi,'MUSIC_calib')
            [filename, pathname]=uiputfile('*.mat','Save results as',['calib\' param.pathnfile{1} '_' param.pathnfile{2} '_2.mat']);
            save([pathname '/' filename],'timbre_calib','timbre_calib_responses');
else
            [filename, pathname]=uiputfile('*.mat','Save results as',['calib\' param.pathnfile{1} '_' param.pathnfile{2} '_2.mat']);
            save([pathname '/' filename],'timbre_calib','timbre_calib_responses','-append');
end
        else
            save(['calib\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'],'timbre_calib','timbre_calib_responses','-append');
        end
    else
        save(['calib\' param.pathnfile{1} '_' param.pathnfile{2} '.mat'],'timbre_calib','timbre_calib_responses');
    end
    close(param.main)   
end