function timbre_training(pathnfile,calib,language,timbre_calib)


%% Settings
param.dif{1} = -1;   % difference for D8 brightness (dB/harmonic)
param.dif{2} = 0.4;  % difference for D9 attack time (sec)
param.dif{3} = -8;   % difference for D10 spectral irregularity (dB)

%% init
if nargin<4,
timbre_calib = [-0.1 -0.5 -0.9 , -1 -4.25 -7.5 ; 0.45 2.04 3.4 , 0.41 1.42 2.02];% these are the calibration values for roll-off and irr
 % first row: x-points 1,2 roll 3,4 irr; second row: respective y-values = compensation amps
end
if nargin<3, language=0; end
if nargin<2, calib=0; end
if nargin<1, pathnfile={'testperson','noHA'}; end

% define default values for target tones
param.pathnfile=pathnfile;
param.calib=calib;
param.language=language;
param.timbre_calib=timbre_calib;

% param.answer: Answers for all trials, 1 right, 0 wrong
% param.playback: the dimension under test (1:roll, 2:dur,
% 3:flux, 4:irr);

%% window design
scsze = get(0,'ScreenSize');
height = 500 ; %480;
length = 680;
pos = [scsze(3)/2-length/2 scsze(4)/2-height/2 length height];

param.main = figure('Name','Timbre: Training','NumberTitle','off');
param.color = get(param.main,'Color');
param.color_active = [0.54 0.7 1];
param.color_activepush = [0.541 0.7 1];
param.color_right = [0.4 0.7 0.2];
param.color_wrong = [1 0.2 0.2];


set(param.main,'MenuBar','none');
set(param.main,'Position', pos);
param.caption = uicontrol('Style', 'text', 'String', 'Timbre: Training', 'FontSize', 14, 'FontWeight', 'bold', 'Position', [length/2 - 180 height-60 360 30],'BackgroundColor',param.color);

param.txt1 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'After pressing "Start", you will hear three signals A, B and X. Please', 'Position', [length/2-300 height-120 600 30], 'BackgroundColor',param.color);
param.txt2 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'decide whether X sounds like A or B by pressing the respective','Position', [length/2-300 height-150 600 30], 'BackgroundColor',param.color);
param.txt3 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'button A or B. Confirm your selection by pressing "Continue".', 'Position', [length/2-300 height-180 600 30], 'BackgroundColor',param.color);
param.txt4 = uicontrol('Style', 'text', 'FontSize', 12, 'HorizontalAlignment','center','String', 'The correct answer will be highlighted in green.', 'Position', [length/2-300 height-210 600 30], 'BackgroundColor',param.color);

if param.language

    set(param.txt1,'String','Nachdem Sie "Start" drcken, hren Sie drei Signale "A", "B" und "X".');
    set(param.txt2,'String','Entscheiden Sie ob "X" wie "A" oder "B" klingt und whlen Sie die ');
    set(param.txt3,'String','entsprechende Antwort. Besttigen Sie Ihre Auswahl mit "Weiter".');
    set(param.txt4,'String','Die richtige Antwort wird grn markiert, ein falsche rot.');
end
   

param.pb_multi=uicontrol('Style','pushbutton','String','Start','tag','1','FontSize', 14, 'FontAngle', 'italic', 'FontWeight', 'bold','Position',[length/2-260 height-470 520 70],'Callback',@function_multi);


note=iconread('images/note.png','png');
sum_note=sum(note,3);
for i = 1:size(note,1)
    for ii=1:size(note,2)
       if sum_note(i,ii)>2.5
           note(i,ii,1:3)=NaN;
       end
    end
end
param.pb_1=uicontrol('Style','pushbutton','String','A    ', 'cdata', note, 'tag', '0', 'FontSize', 32, 'Position', [length/2-260 height-360 120 120], 'Callback',@function_pb);
param.pb_2=uicontrol('Style','pushbutton','String','B    ', 'cdata', note, 'tag', '0', 'FontSize', 32, 'Position', [length/2-60 height-360 120 120], 'Callback',@function_pb);
param.pb_3=uicontrol('Style','pushbutton','String','X    ', 'HorizontalAlignment', 'left', 'cdata', note, 'tag', '0', 'FontSize', 32, 'Position', [length/2+140 height-360 120 120]);
param.color_pb=get(param.pb_1,'BackgroundColor');


%% stimulus design
param.fs=44100;
param.dur = 1;
param.roll = -2;
param.f0 = 220;

param.Nharm = 40;
t = 0:1/param.fs:param.dur-1/param.fs;
T = t'*(1:param.Nharm);

load('randparam.mat')
param.phase=repmat(randparam.phase(1:param.Nharm),size(T,1),1);

% define standard tone
param.attack_0 = 0.04;
param.fadeout = 0.08;
param.counter=1;
param.playback= [1 2 3];
param.playbackmode=[randperm(4);randperm(4);randperm(4)];


% calculation of the standard tone
REF = 0.01 * sin(2*pi*param.f0*T+param.phase);
harm = ones(1,param.Nharm);
for i = 2:param.Nharm
    harm(i) = (10^(param.roll/20))^(i-1);
end
REF = REF.*repmat(harm,size(T,1),1).*(ones(size(T)));
ref = sum(REF,2);

% fade-in and -out
attack_samples = 1:round(param.fs*param.attack_0);
attack = attack_samples/attack_samples(end); % linear attack
ref(attack_samples) = ref(attack_samples).*attack';
% standard fade out
fadeout_samples = 1:round(param.fs*param.fadeout);
fadeout = (fadeout_samples/fadeout_samples(end));
ref(end-fadeout_samples(end)+1:end) = ref(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
param.ref=[ref;zeros(round(param.fs*0.4),1)];

% preparation target tone for first round
switch param.playback(param.counter)
    case 1
        roll=10^((param.roll+param.dif{1}(end))/20);
        for i = 2:param.Nharm
            harm(i) = roll^(i-1);
        end
        SIG = 0.01 * sin(2*pi*param.f0*T + param.phase);
        SIG = SIG.*repmat(harm,size(T,1),1); %.*(ones(size(T)));
        sig = sum(SIG,2);
        attack_samples = 1:round(param.fs*param.attack_0);
        attack = (attack_samples/attack_samples(end)); % linear attack
        sig(attack_samples) = sig(attack_samples).*attack';
        % standard fade out
        fadeout_samples = 1:round(param.fs*param.fadeout);
        fadeout = (fadeout_samples/fadeout_samples(end));
        sig(end-fadeout_samples(end)+1:end) = sig(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
        
        % roll off compensation
        if param.dif{1}(end)>param.timbre_calib(1,1)   % values are negative, so magnitude is reversed
            amp=param.timbre_calib(2,1)*(param.dif{1}(end))/(param.timbre_calib(1,1));
        else if param.dif{1}(end)>param.timbre_calib(1,2)
                amp=param.timbre_calib(2,1)+(param.dif{1}(end)-param.timbre_calib(1,1))/(param.timbre_calib(1,2)-param.timbre_calib(1,1))*(param.timbre_calib(2,2)-param.timbre_calib(2,1));
            else 
                amp=param.timbre_calib(2,2)+(param.dif{1}(end)-param.timbre_calib(1,2))/(param.timbre_calib(1,3)-param.timbre_calib(1,2))*(param.timbre_calib(2,3)-param.timbre_calib(2,2));
            end
        end
        param.sig=[10^(amp/20)*sig;zeros(round(param.fs*0.4),1)];
        
    case 2
        for i = 2:param.Nharm
            harm(i) = (10^(param.roll/20)^(i-1));
        end
        SIG = 0.01 * sin(2*pi*param.f0*T + param.phase);
        SIG = SIG.*repmat(harm,size(T,1),1);
        sig = sum(SIG,2);
        attack_samples = 1:round(param.fs*(param.attack_0+param.dif{2}(end)));
        attack = (attack_samples/attack_samples(end));
        sig(attack_samples) = sig(attack_samples).*attack';
        % standard fade out
        fadeout_samples = 1:round(param.fs*param.fadeout);
        fadeout = (fadeout_samples/fadeout_samples(end));
        sig(end-fadeout_samples(end)+1:end) = sig(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
        param.sig=[sig;zeros(round(param.fs*0.4),1)];
        
    case 3
        for i = 2:param.Nharm
            harm(i) = (10^(param.roll/20)^(i-1));
        end
        for i = 1:floor(param.Nharm/2)
            harm(2*i)=harm(2*i)*10^(param.dif{3}(end)/20);
        end
        SIG = 0.01 * sin(2*pi*param.f0*T + param.phase);
        SIG = SIG.*repmat(harm,size(T,1),1);
        sig = sum(SIG,2);
        attack_samples = 1:round(param.fs*param.attack_0);
        attack = (attack_samples/attack_samples(end));
        sig(attack_samples) = sig(attack_samples).*attack';
        % standard fade out
        fadeout_samples = 1:round(param.fs*param.fadeout);
        fadeout = (fadeout_samples/fadeout_samples(end));
        sig(end-fadeout_samples(end)+1:end) = sig(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
        
% irr compensation
        if param.dif{3}(end)>param.timbre_calib(1,4)   % values are negative, so magnitude is reversed
            amp=param.timbre_calib(2,4)*(param.dif{3}(end))/(param.timbre_calib(1,4));
        else if param.dif{3}(end)>param.timbre_calib(1,5)
                amp=param.timbre_calib(2,4)+(param.dif{3}(end)-param.timbre_calib(1,4))/(param.timbre_calib(1,5)-param.timbre_calib(1,4))*(param.timbre_calib(2,5)-param.timbre_calib(2,4));
            else 
                amp=param.timbre_calib(2,5)+(param.dif{3}(end)-param.timbre_calib(1,5))/(param.timbre_calib(1,6)-param.timbre_calib(1,5))*(param.timbre_calib(2,6)-param.timbre_calib(2,5));
            end
        end
        param.sig=[10^(amp/20)*sig;zeros(round(param.fs*0.4),1)];
end

param.answer={0,0,0};
guidata(param.main,param)

%%%%---------------------------GUI functions--------------------------%%%%
function function_pb(hObject,eventdata)

param=guidata(hObject);
if ~(sum(get(param.pb_1,'BackgroundColor')==param.color_active)==3 || sum(get(param.pb_2,'BackgroundColor')==param.color_active)==3 || sum(get(param.pb_3,'BackgroundColor')==param.color_active)==3)
    set(param.pb_1,'tag','0','BackgroundColor',param.color_pb);
    set(param.pb_2,'tag','0','BackgroundColor',param.color_pb);
    set(hObject,'tag','1','BackgroundColor',param.color_activepush);
    set(param.pb_3,'String',get(hObject,'String'));
    guidata(param.main,param);
end
guidata(param.main,param)

function function_multi(hObject,eventdata)
param=guidata(hObject);

% only first time to change the String
if strcmp(get(hObject,'String'),'Start')
    tic;
    set(hObject,'String','Continue','enable','off');
        if param.language
        set(hObject,'String','Weiter')
    end
    set(param.pb_1,'BackgroundColor',param.color_pb);
    set(param.pb_2,'BackgroundColor',param.color_pb);
    set(param.pb_3,'String','X    ');
    if max(abs([param.ref; param.sig]))<0.3
        switch param.playbackmode(param.playback(param.counter),ceil(param.counter/3))
            case 1
                out1 = param.ref;
                out2 = param.sig;
                out3 = param.ref(1:end-round(param.fs*0.4)+1);
            case 2
                out1 = param.ref;
                out2 = param.sig;
                out3 = param.sig(1:end-round(param.fs*0.4)+1);
            case 3
                out1 = param.sig;
                out2 = param.ref;
                out3 = param.sig(1:end-round(param.fs*0.4)+1);
            case 4
                out1 = param.sig;
                out2 = param.ref;
                out3 = param.ref(1:end-round(param.fs*0.4)+1);
        end
        
        out1 = out1*10^(param.calib/20);
        out2 = out2*10^(param.calib/20);
        out3 = out3*10^(param.calib/20);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_active);
        param.player=audioplayer(out1,param.fs);
        guidata(param.main,param)
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_1,'BackgroundColor',param.color_pb);
        set(param.pb_2,'BackgroundColor',param.color_active);
        set(param.pb_3,'String','X    ');
        param.player=audioplayer(out2,param.fs);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_2,'BackgroundColor',param.color_pb);
        set(param.pb_3,'BackgroundColor',param.color_active);
        set(param.pb_3,'String','X    ');
        param.player=audioplayer(out3,param.fs);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_3,'BackgroundColor',param.color_pb);
        set(hObject,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
    return
end

if strcmp(get(param.pb_3,'String'),'X    ')
    return
end
set(hObject,'enable','off')

% return if no answer is selected ( not needed as "Confirm button" is desactivated

% define whether the answer is right(1) or wrong(0)
if str2num(get(param.pb_1,'tag'))== mod(param.playbackmode(param.playback(param.counter),ceil(param.counter/3)),2) % switch 1 and 3 is right when left button(1)
    param.answer{param.playback(1,param.counter)}(ceil(param.counter/3)) = 1;
    if str2num(get(param.pb_1,'tag'))==1
        set(param.pb_1,'BackgroundColor',param.color_right);
    else
        set(param.pb_2,'BackgroundColor',param.color_right);
    end
else
    param.answer{param.playback(1,param.counter)}(ceil(param.counter/3)) = 0;
    if str2num(get(param.pb_1,'tag'))==1
        set(param.pb_1,'BackgroundColor',param.color_wrong);
    else
        set(param.pb_2,'BackgroundColor',param.color_wrong);
    end
end
pause(1.5)

%% add new 4-element playback sequence when the counter is at the end
if mod(param.counter,3) == 0
    add = randperm(3);
    if param.playback(end) == add(1,1)
        add(1,1) = add(1,2);
        add(1,2) = param.playback(1,end);
    end
    param.playback=[param.playback add];
end

%% add new 6-element playbackbtn sequence when the counter is at the end
if mod(param.counter,12) == 0
    helper=[randperm(4);randperm(4);randperm(4)];
    param.playbackmode=[param.playbackmode, helper];
end

%% calculation and playback of the signal of next trial
param.counter=param.counter+1;
t = 0:1/param.fs:param.dur-1/param.fs;
T = t'*(1:param.Nharm);
harm = ones(1,param.Nharm);

switch param.playback(param.counter)
    case 1
        roll=10^((param.roll+param.dif{1}(end))/20);
        for i = 2:param.Nharm
            harm(i) = roll^(i-1);
        end
        SIG = 0.01 * sin(2*pi*param.f0*T + param.phase);
        SIG = SIG.*repmat(harm,size(T,1),1);
        sig = sum(SIG,2);
        attack_samples = 1:round(param.fs*param.attack_0);
        attack = (attack_samples/attack_samples(end));
        sig(attack_samples) = sig(attack_samples).*attack';
        % standard fade out
        fadeout_samples = 1:round(param.fs*param.fadeout);
        fadeout = (fadeout_samples/fadeout_samples(end));
        sig(end-fadeout_samples(end)+1:end) = sig(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
        
        % roll off compensation
        if param.dif{1}(end)>param.timbre_calib(1,1)   % values are negative, so magnitude is reversed
            amp=param.timbre_calib(2,1)*(param.dif{1}(end))/(param.timbre_calib(1,1));
        else if param.dif{1}(end)>param.timbre_calib(1,2)
                amp=param.timbre_calib(2,1)+(param.dif{1}(end)-param.timbre_calib(1,1))/(param.timbre_calib(1,2)-param.timbre_calib(1,1))*(param.timbre_calib(1,2)-param.timbre_calib(1,1));
            else
                amp=param.dif{1}(end) * param.timbre_calib(2,2) / param.timbre_calib(1,2);
            end
        end
        param.sig=[10^(amp/20)*sig;zeros(round(param.fs*0.4),1)];
        
    case 2
        for i = 2:param.Nharm
            harm(i) = (10^(param.roll/20)^(i-1));
        end
        SIG = 0.01 * sin(2*pi*param.f0*T + param.phase);
        SIG = SIG.*repmat(harm,size(T,1),1);
        sig = sum(SIG,2);
        attack_samples = 1:round(param.fs*(param.attack_0+param.dif{2}(end)));
        attack = (attack_samples/attack_samples(end));
        sig(attack_samples) = sig(attack_samples).*attack';
        % standard fade out
        fadeout_samples = 1:round(param.fs*param.fadeout);
        fadeout = (fadeout_samples/fadeout_samples(end));
        sig(end-fadeout_samples(end)+1:end) = sig(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
        param.sig=[sig;zeros(round(param.fs*0.4),1)];
        
    case 3
        for i = 2:param.Nharm
            harm(i) = (10^(param.roll/20)^(i-1));
        end
        for i = 1:floor(param.Nharm/2)
            harm(2*i)=harm(2*i)*10^(param.dif{3}(end)/20);
        end
        SIG = 0.01 * sin(2*pi*param.f0*T + param.phase);
        SIG = SIG.*repmat(harm,size(T,1),1);
        sig = sum(SIG,2);
        attack_samples = 1:round(param.fs*param.attack_0);
        attack = (attack_samples/attack_samples(end));
        sig(attack_samples) = sig(attack_samples).*attack';
        % standard fade out
        fadeout_samples = 1:round(param.fs*param.fadeout);
        fadeout = (fadeout_samples/fadeout_samples(end));
        sig(end-fadeout_samples(end)+1:end) = sig(end-fadeout_samples(end)+1:end).*fadeout(end:-1:1)';
        
% irr compensation
        if param.dif{3}(end)>param.timbre_calib(1,4)   % values are negative, so magnitude is reversed
            amp=param.timbre_calib(2,4)*(param.dif{3}(end))/(param.timbre_calib(1,4));
        else if param.dif{3}(end)>param.timbre_calib(1,5)
                amp=param.timbre_calib(2,4)+(param.dif{3}(end)-param.timbre_calib(1,4))/(param.timbre_calib(1,5)-param.timbre_calib(1,4))*(param.timbre_calib(2,5)-param.timbre_calib(2,4));
            else 
                amp=param.timbre_calib(2,5)+(param.dif{3}(end)-param.timbre_calib(1,5))/(param.timbre_calib(1,6)-param.timbre_calib(1,5))*(param.timbre_calib(2,6)-param.timbre_calib(2,5));
            end
        end
        param.sig=[10^(amp/20)*sig;zeros(round(param.fs*0.4),1)];
end


set(param.pb_1,'BackgroundColor',param.color_pb);
set(param.pb_2,'BackgroundColor',param.color_pb);
set(param.pb_3,'BackgroundColor',param.color_pb,'String','X    ');
param.time(param.counter-1)=toc;
if param.counter<=6
    if max(abs([param.ref; param.sig]))<0.3
        switch param.playbackmode(param.playback(param.counter),ceil(param.counter/3))
            case 1
                out1 = param.ref;
                out2 = param.sig;
                out3 = param.ref(1:end-round(param.fs*0.4)+1);
            case 2
                out1 = param.ref;
                out2 = param.sig;
                out3 = param.sig(1:end-round(param.fs*0.4)+1);
            case 3
                out1 = param.sig;
                out2 = param.ref;
                out3 = param.sig(1:end-round(param.fs*0.4)+1);
            case 4
                out1 = param.sig;
                out2 = param.ref;
                out3 = param.ref(1:end-round(param.fs*0.4)+1);
        end
        out1 = out1*10^(param.calib/20);
        out2 = out2*10^(param.calib/20);
        out3 = out3*10^(param.calib/20);
        pause(0.5)
        set(param.pb_1,'BackgroundColor',param.color_active);
        set(param.pb_3,'String','X    ');
        param.player=audioplayer(out1,param.fs);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_1,'BackgroundColor',param.color_pb);
        set(param.pb_2,'BackgroundColor',param.color_active);
        set(param.pb_3,'String','X    ');
        param.player=audioplayer(out2,param.fs);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_2,'BackgroundColor',param.color_pb);
        set(param.pb_3,'BackgroundColor',param.color_active);
        set(param.pb_3,'String','X    ');
        param.player=audioplayer(out3,param.fs);
        pause(0.001)
        playblocking(param.player);
        pause(0.001)
        set(param.pb_3,'BackgroundColor',param.color_pb);
        set(hObject,'enable','on');
    else
        disp('Warning: Signal too loud and not played back for safety reasons!')
    end
    guidata(param.main,param)
else
    % create folder if it doesn't exist
    help1=dir;
    help2=0;
    for ii=1:size(help1,1)
        if strcmp('results',help1(ii).name)
            help2=help2+1;
        end
    end
    if help2==0;
        mkdir('results')
    end
    
    close(param.main)
end